@@ -142,19 +142,25 @@ static int add_last_error(PyObject* self, buffer_t buffer,
 }
 
 static int init_insert_buffer(buffer_t buffer, int request_id, int options,
-                              const char* coll_name, int coll_name_len) {
-    /* Save space for message length */
-    int length_location = buffer_save_space(buffer, 4);
-    if (length_location == -1) {
-        PyErr_NoMemory();
-        return length_location;
+                              const char* coll_name, int coll_name_len,
+                              int compress) {
+    int length_location = 0;
+    if (!compress) {
+        /* Save space for message length */
+        int length_location = buffer_save_space(buffer, 4);
+        if (length_location == -1) {
+            PyErr_NoMemory();
+            return length_location;
+        }
+        if (!buffer_write_int32(buffer, (int32_t)request_id) ||
+            !buffer_write_bytes(buffer,
+                                "\x00\x00\x00\x00"
+                                "\xd2\x07\x00\x00",
+                                8)) {
+            return -1;
+        }
     }
-    if (!buffer_write_int32(buffer, (int32_t)request_id) ||
-        !buffer_write_bytes(buffer,
-                            "\x00\x00\x00\x00"
-                            "\xd2\x07\x00\x00",
-                            8) ||
-        !buffer_write_int32(buffer, (int32_t)options) ||
+    if (!buffer_write_int32(buffer, (int32_t)options) ||
         !buffer_write_bytes(buffer,
                             coll_name,
                             coll_name_len + 1)) {
@@ -211,7 +217,8 @@ static PyObject* _cbson_insert_message(PyObject* self, PyObject* args) {
                                          request_id,
                                          flags,
                                          collection_name,
-                                         collection_name_length);
+                                         collection_name_length,
+                                         0);
     if (length_location == -1) {
         destroy_codec_options(&options);
         PyMem_Free(collection_name);
@@ -639,6 +646,127 @@ static PyObject* _cbson_get_more_message(PyObject* self, PyObject* args) {
     return result;
 }
 
+/*
+ * NOTE this method handles multiple documents in a type one payload but
+ * it does not perform batch splitting and the total message size is
+ * only checked *after* generating the entire message.
+ */
+static PyObject* _cbson_op_msg(PyObject* self, PyObject* args) {
+    struct module_state *state = GETSTATE(self);
+
+    /* NOTE just using a random number as the request_id */
+    int request_id = rand();
+    unsigned int flags;
+    PyObject* command;
+    char* identifier = NULL;
+    int identifier_length = 0;
+    PyObject* docs;
+    PyObject* doc;
+    unsigned char check_keys = 0;
+    codec_options_t options;
+    buffer_t buffer;
+    int length_location, message_length;
+    int total_size = 0;
+    int max_doc_size = 0;
+    PyObject* result = NULL;
+    PyObject* iterator = NULL;
+
+    /*flags, command, identifier, docs, check_keys, opts*/
+    if (!PyArg_ParseTuple(args, "IOet#ObO&",
+                          &flags,
+                          &command,
+                          "utf-8",
+                          &identifier,
+                          &identifier_length,
+                          &docs,
+                          &check_keys,
+                          convert_codec_options, &options)) {
+        return NULL;
+    }
+    buffer = buffer_new();
+    if (!buffer) {
+        PyErr_NoMemory();
+        goto bufferfail;
+    }
+
+    // save space for message length
+    length_location = buffer_save_space(buffer, 4);
+    if (length_location == -1) {
+        PyErr_NoMemory();
+        goto bufferfail;
+    }
+    if (!buffer_write_int32(buffer, (int32_t)request_id) ||
+        !buffer_write_bytes(buffer,
+                            "\x00\x00\x00\x00" /* responseTo */
+                            "\xdd\x07\x00\x00" /* 2013 */, 8)) {
+        goto encodefail;
+    }
+
+    if (!buffer_write_int32(buffer, (int32_t)flags) ||
+        !buffer_write_bytes(buffer, "\x00", 1) /* Payload type 0 */) {
+        goto encodefail;
+    }
+    total_size = write_dict(state->_cbson, buffer, command, 0,
+                          &options, 1);
+    if (!total_size) {
+        goto encodefail;
+    }
+
+    if (identifier_length) {
+        int payload_one_length_location, payload_length;
+        /* Payload type 1 */
+        if (!buffer_write_bytes(buffer, "\x01", 1)) {
+            goto encodefail;
+        }
+        /* save space for payload 0 length */
+        payload_one_length_location = buffer_save_space(buffer, 4);
+        /* C string identifier */
+        if (!buffer_write_bytes(buffer, identifier, identifier_length + 1)) {
+            goto encodefail;
+        }
+        iterator = PyObject_GetIter(docs);
+        if (iterator == NULL) {
+            goto encodefail;
+        }
+        while ((doc = PyIter_Next(iterator)) != NULL) {
+            int encoded_doc_size = write_dict(
+                      state->_cbson, buffer, doc, check_keys,
+                      &options, 1);
+            if (!encoded_doc_size) {
+                Py_CLEAR(doc);
+                goto encodefail;
+            }
+            if (encoded_doc_size > max_doc_size) {
+                max_doc_size = encoded_doc_size;
+            }
+            Py_CLEAR(doc);
+        }
+
+        payload_length = buffer_get_position(buffer) - payload_one_length_location;
+        buffer_write_int32_at_position(
+            buffer, payload_one_length_location, (int32_t)payload_length);
+        total_size += payload_length;
+    }
+
+    message_length = buffer_get_position(buffer) - length_location;
+    buffer_write_int32_at_position(
+        buffer, length_location, (int32_t)message_length);
+
+    /* objectify buffer */
+    result = Py_BuildValue("i" BYTES_FORMAT_STRING "ii", request_id,
+                           buffer_get_buffer(buffer),
+                           buffer_get_position(buffer),
+                           total_size,
+                           max_doc_size);
+encodefail:
+    Py_XDECREF(iterator);
+    buffer_free(buffer);
+bufferfail:
+    destroy_codec_options(&options);
+    return result;
+}
+
+
 static void
 _set_document_too_large(int size, long max) {
     PyObject* DocumentTooLarge = _error("DocumentTooLarge");
@@ -660,7 +788,7 @@ static PyObject*
 _send_insert(PyObject* self, PyObject* ctx,
              PyObject* gle_args, buffer_t buffer,
              char* coll_name, int coll_len, int request_id, int safe,
-             codec_options_t* options, PyObject* to_publish) {
+             codec_options_t* options, PyObject* to_publish, int compress) {
 
     if (safe) {
         if (!add_last_error(self, buffer, request_id,
@@ -669,16 +797,16 @@ _send_insert(PyObject* self, PyObject* ctx,
         }
     }
 
-    /* The max_doc_size parameter for legacy_write is the max size of any
-     * document in buffer. We enforced max size already, pass 0 here. */
-    return PyObject_CallMethod(ctx, "legacy_write",
-                               "i" BYTES_FORMAT_STRING "iNO",
+    /* The max_doc_size parameter for legacy_bulk_insert is the max size of
+     * any document in buffer. We enforced max size already, pass 0 here. */
+    return PyObject_CallMethod(ctx, "legacy_bulk_insert",
+                               "i" BYTES_FORMAT_STRING "iNOi",
                                request_id,
                                buffer_get_buffer(buffer),
                                buffer_get_position(buffer),
                                0,
                                PyBool_FromLong((long)safe),
-                               to_publish);
+                               to_publish, compress);
 }
 
 static PyObject* _cbson_do_batched_insert(PyObject* self, PyObject* args) {
@@ -689,6 +817,7 @@ static PyObject* _cbson_do_batched_insert(PyObject* self, PyObject* args) {
     int send_safe, flags = 0;
     int length_location, message_length;
     int collection_name_length;
+    int compress;
     char* collection_name = NULL;
     PyObject* docs;
     PyObject* doc;
@@ -698,6 +827,7 @@ static PyObject* _cbson_do_batched_insert(PyObject* self, PyObject* args) {
     PyObject* result;
     PyObject* max_bson_size_obj;
     PyObject* max_message_size_obj;
+    PyObject* compress_obj;
     PyObject* to_publish = NULL;
     unsigned char check_keys;
     unsigned char safe;
@@ -754,6 +884,17 @@ static PyObject* _cbson_do_batched_insert(PyObject* self, PyObject* args) {
         return NULL;
     }
 
+    compress_obj = PyObject_GetAttrString(ctx, "compress");
+    compress = PyObject_IsTrue(compress_obj);
+    Py_XDECREF(compress_obj);
+    if (compress == -1) {
+        destroy_codec_options(&options);
+        PyMem_Free(collection_name);
+        return NULL;
+    }
+
+    compress = compress && !(safe || send_safe);
+
     buffer = buffer_new();
     if (!buffer) {
         destroy_codec_options(&options);
@@ -766,7 +907,8 @@ static PyObject* _cbson_do_batched_insert(PyObject* self, PyObject* args) {
                                          request_id,
                                          flags,
                                          collection_name,
-                                         collection_name_length);
+                                         collection_name_length,
+                                         compress);
     if (length_location == -1) {
         goto insertfail;
     }
@@ -797,13 +939,15 @@ static PyObject* _cbson_do_batched_insert(PyObject* self, PyObject* args) {
             /* If we've encoded anything send it before raising. */
             if (!empty) {
                 buffer_update_position(buffer, before);
-                message_length = buffer_get_position(buffer) - length_location;
-                buffer_write_int32_at_position(
-                    buffer, length_location, (int32_t)message_length);
+                if (!compress) {
+                    message_length = buffer_get_position(buffer) - length_location;
+                    buffer_write_int32_at_position(
+                        buffer, length_location, (int32_t)message_length);
+                }
                 result = _send_insert(self, ctx, last_error_args, buffer,
                                       collection_name, collection_name_length,
                                       request_id, send_safe, &options,
-                                      to_publish);
+                                      to_publish, compress);
                 if (!result)
                     goto iterfail;
                 Py_DECREF(result);
@@ -826,7 +970,8 @@ static PyObject* _cbson_do_batched_insert(PyObject* self, PyObject* args) {
                                                new_request_id,
                                                flags,
                                                collection_name,
-                                               collection_name_length);
+                                               collection_name_length,
+                                               compress);
             if (message_start == -1) {
                 buffer_free(new_buffer);
                 goto iterfail;
@@ -841,13 +986,16 @@ static PyObject* _cbson_do_batched_insert(PyObject* self, PyObject* args) {
 
             /* Roll back to the beginning of this document. */
             buffer_update_position(buffer, before);
-            message_length = buffer_get_position(buffer) - length_location;
-            buffer_write_int32_at_position(
-                buffer, length_location, (int32_t)message_length);
+            if (!compress) {
+                message_length = buffer_get_position(buffer) - length_location;
+                buffer_write_int32_at_position(
+                    buffer, length_location, (int32_t)message_length);
+            }
 
             result = _send_insert(self, ctx, last_error_args, buffer,
                                   collection_name, collection_name_length,
-                                  request_id, send_safe, &options, to_publish);
+                                  request_id, send_safe, &options, to_publish,
+                                  compress);
 
             buffer_free(buffer);
             buffer = new_buffer;
@@ -927,14 +1075,16 @@ static PyObject* _cbson_do_batched_insert(PyObject* self, PyObject* args) {
         goto insertfail;
     }
 
-    message_length = buffer_get_position(buffer) - length_location;
-    buffer_write_int32_at_position(
-        buffer, length_location, (int32_t)message_length);
+    if (!compress) {
+        message_length = buffer_get_position(buffer) - length_location;
+        buffer_write_int32_at_position(
+            buffer, length_location, (int32_t)message_length);
+    }
 
     /* Send the last (or only) batch */
     result = _send_insert(self, ctx, last_error_args, buffer,
                           collection_name, collection_name_length,
-                          request_id, safe, &options, to_publish);
+                          request_id, safe, &options, to_publish, compress);
 
     Py_DECREF(to_publish);
     PyMem_Free(collection_name);
@@ -971,75 +1121,32 @@ insertfail:
     return NULL;
 }
 
-static buffer_t
-_command_buffer_new(char* ns, int ns_len) {
-    buffer_t buffer;
-    if (!(buffer = buffer_new())) {
-        PyErr_NoMemory();
-        return NULL;
-    }
-    /* Save space for message length and request id */
-    if ((buffer_save_space(buffer, 8)) == -1) {
-        PyErr_NoMemory();
-        buffer_free(buffer);
-        return NULL;
-    }
-    if (!buffer_write_bytes(buffer,
-                            "\x00\x00\x00\x00"  /* responseTo */
-                            "\xd4\x07\x00\x00"  /* opcode */
-                            "\x00\x00\x00\x00", /* flags */
-                            12) ||
-        !buffer_write_bytes(buffer,
-                            ns, ns_len + 1) ||  /* namespace */
-        !buffer_write_bytes(buffer,
-                            "\x00\x00\x00\x00"  /* skip */
-                            "\xFF\xFF\xFF\xFF", /* limit (-1) */
-                            8)) {
-        buffer_free(buffer);
-        return NULL;
-    }
-    return buffer;
-}
-
 #define _INSERT 0
 #define _UPDATE 1
 #define _DELETE 2
 
-static PyObject*
-_cbson_do_batched_write_command(PyObject* self, PyObject* args) {
-    struct module_state *state = GETSTATE(self);
+/* OP_MSG ----------------------------------------------- */
+
+static int
+_batched_op_msg(
+        unsigned char op, unsigned char check_keys, unsigned char ack,
+        PyObject* command, PyObject* docs, PyObject* ctx,
+        PyObject* to_publish, codec_options_t options,
+        buffer_t buffer, struct module_state *state) {
 
     long max_bson_size;
-    long max_cmd_size;
     long max_write_batch_size;
+    long max_message_size;
     int idx = 0;
-    int cmd_len_loc;
-    int lst_len_loc;
-    int ns_len;
-    int request_id;
+    int size_location;
     int position;
     int length;
-    char *ns = NULL;
     PyObject* max_bson_size_obj;
     PyObject* max_write_batch_size_obj;
-    PyObject* command;
+    PyObject* max_message_size_obj;
     PyObject* doc;
-    PyObject* docs;
-    PyObject* ctx;
     PyObject* iterator;
-    PyObject* result;
-    PyObject* to_publish = NULL;
-    unsigned char op;
-    unsigned char check_keys;
-    codec_options_t options;
-    buffer_t buffer;
-
-    if (!PyArg_ParseTuple(args, "et#bOObO&O", "utf-8",
-                          &ns, &ns_len, &op, &command, &docs, &check_keys,
-                          convert_codec_options, &options,
-                          &ctx)) {
-        return NULL;
-    }
+    char* flags = ack ? "\x00\x00\x00\x00" : "\x02\x00\x00\x00";
 
     max_bson_size_obj = PyObject_GetAttrString(ctx, "max_bson_size");
 #if PY_MAJOR_VERSION >= 3
@@ -1049,10 +1156,330 @@ _cbson_do_batched_write_command(PyObject* self, PyObject* args) {
 #endif
     Py_XDECREF(max_bson_size_obj);
     if (max_bson_size == -1) {
-        destroy_codec_options(&options);
-        PyMem_Free(ns);
+        return 0;
+    }
+
+    max_write_batch_size_obj = PyObject_GetAttrString(ctx, "max_write_batch_size");
+#if PY_MAJOR_VERSION >= 3
+    max_write_batch_size = PyLong_AsLong(max_write_batch_size_obj);
+#else
+    max_write_batch_size = PyInt_AsLong(max_write_batch_size_obj);
+#endif
+    Py_XDECREF(max_write_batch_size_obj);
+    if (max_write_batch_size == -1) {
+        return 0;
+    }
+
+    max_message_size_obj = PyObject_GetAttrString(ctx, "max_message_size");
+#if PY_MAJOR_VERSION >= 3
+    max_message_size = PyLong_AsLong(max_message_size_obj);
+#else
+    max_message_size = PyInt_AsLong(max_message_size_obj);
+#endif
+    Py_XDECREF(max_message_size_obj);
+    if (max_message_size == -1) {
+        return 0;
+    }
+
+    if (!buffer_write_bytes(buffer, flags, 4)) {
+        return 0;
+    }
+    /* Type 0 Section */
+    if (!buffer_write_bytes(buffer, "\x00", 1)) {
+        return 0;
+    }
+    if (!write_dict(state->_cbson, buffer, command, 0,
+                    &options, 0)) {
+        return 0;
+    }
+
+    /* Type 1 Section */
+    if (!buffer_write_bytes(buffer, "\x01", 1)) {
+        return 0;
+    }
+    /* Save space for size */
+    size_location = buffer_save_space(buffer, 4);
+    if (size_location == -1) {
+        PyErr_NoMemory();
+        return 0;
+    }
+
+    switch (op) {
+    case _INSERT:
+        {
+            if (!buffer_write_bytes(buffer, "documents\x00", 10))
+                goto cmdfail;
+            break;
+        }
+    case _UPDATE:
+        {
+            /* MongoDB does key validation for update. */
+            check_keys = 0;
+            if (!buffer_write_bytes(buffer, "updates\x00", 8))
+                goto cmdfail;
+            break;
+        }
+    case _DELETE:
+        {
+            /* Never check keys in a delete command. */
+            check_keys = 0;
+            if (!buffer_write_bytes(buffer, "deletes\x00", 8))
+                goto cmdfail;
+            break;
+        }
+    default:
+        {
+            PyObject* InvalidOperation = _error("InvalidOperation");
+            if (InvalidOperation) {
+                PyErr_SetString(InvalidOperation, "Unknown command");
+                Py_DECREF(InvalidOperation);
+            }
+            return 0;
+        }
+    }
+
+    iterator = PyObject_GetIter(docs);
+    if (iterator == NULL) {
+        PyObject* InvalidOperation = _error("InvalidOperation");
+        if (InvalidOperation) {
+            PyErr_SetString(InvalidOperation, "input is not iterable");
+            Py_DECREF(InvalidOperation);
+        }
+        return 0;
+    }
+    while ((doc = PyIter_Next(iterator)) != NULL) {
+        int cur_doc_begin = buffer_get_position(buffer);
+        int cur_size;
+        int doc_too_large = 0;
+        int unacked_doc_too_large = 0;
+        if (!write_dict(state->_cbson, buffer, doc, check_keys,
+                        &options, 1)) {
+            goto cmditerfail;
+        }
+        cur_size = buffer_get_position(buffer) - cur_doc_begin;
+
+        /* Does the first document exceed max_message_size? */
+        doc_too_large = (idx == 0 && (buffer_get_position(buffer) > max_message_size));
+        /* When OP_MSG is used unacknowledged we have to check
+         * document size client side or applications won't be notified.
+         * Otherwise we let the server deal with documents that are too large
+         * since ordered=False causes those documents to be skipped instead of
+         * halting the bulk write operation.
+         * */
+        unacked_doc_too_large = (!ack && cur_size > max_bson_size);
+        if (doc_too_large || unacked_doc_too_large) {
+            if (op == _INSERT) {
+                _set_document_too_large(cur_size, max_bson_size);
+            } else {
+                PyObject* DocumentTooLarge = _error("DocumentTooLarge");
+                if (DocumentTooLarge) {
+                    /*
+                     * There's nothing intelligent we can say
+                     * about size for update and delete.
+                     */
+                    PyErr_Format(
+                        DocumentTooLarge,
+                        "%s command document too large",
+                        (op == _UPDATE) ? "update": "delete");
+                    Py_DECREF(DocumentTooLarge);
+                }
+            }
+            goto cmditerfail;
+        }
+        /* We have enough data, return this batch. */
+        if (buffer_get_position(buffer) > max_message_size) {
+            /*
+             * Roll the existing buffer back to the beginning
+             * of the last document encoded.
+             */
+            buffer_update_position(buffer, cur_doc_begin);
+            break;
+        }
+        if (PyList_Append(to_publish, doc) < 0) {
+            goto cmditerfail;
+        }
+        Py_CLEAR(doc);
+        idx += 1;
+        /* We have enough documents, return this batch. */
+        if (idx == max_write_batch_size) {
+            break;
+        }
+    }
+    Py_DECREF(iterator);
+
+    if (PyErr_Occurred()) {
+        goto cmdfail;
+    }
+
+    position = buffer_get_position(buffer);
+    length = position - size_location;
+    buffer_write_int32_at_position(buffer, size_location, (int32_t)length);
+    return 1;
+
+cmditerfail:
+    Py_XDECREF(doc);
+    Py_DECREF(iterator);
+cmdfail:
+    return 0;
+}
+
+static PyObject*
+_cbson_encode_batched_op_msg(PyObject* self, PyObject* args) {
+    unsigned char op;
+    unsigned char check_keys;
+    unsigned char ack;
+    PyObject* command;
+    PyObject* docs;
+    PyObject* ctx = NULL;
+    PyObject* to_publish = NULL;
+    PyObject* result = NULL;
+    codec_options_t options;
+    buffer_t buffer;
+    struct module_state *state = GETSTATE(self);
+
+    if (!PyArg_ParseTuple(args, "bOObbO&O",
+                          &op, &command, &docs, &check_keys, &ack,
+                          convert_codec_options, &options,
+                          &ctx)) {
         return NULL;
     }
+    if (!(buffer = buffer_new())) {
+        PyErr_NoMemory();
+        destroy_codec_options(&options);
+        return NULL;
+    }
+    if (!(to_publish = PyList_New(0))) {
+        goto fail;
+    }
+
+    if (!_batched_op_msg(
+            op,
+            check_keys,
+            ack,
+            command,
+            docs,
+            ctx,
+            to_publish,
+            options,
+            buffer,
+            state)) {
+        goto fail;
+    }
+
+    result = Py_BuildValue(BYTES_FORMAT_STRING "O",
+                           buffer_get_buffer(buffer),
+                           buffer_get_position(buffer),
+                           to_publish);
+fail:
+    destroy_codec_options(&options);
+    buffer_free(buffer);
+    Py_XDECREF(to_publish);
+    return result;
+}
+
+static PyObject*
+_cbson_batched_op_msg(PyObject* self, PyObject* args) {
+    unsigned char op;
+    unsigned char check_keys;
+    unsigned char ack;
+    int request_id;
+    int position;
+    PyObject* command;
+    PyObject* docs;
+    PyObject* ctx = NULL;
+    PyObject* to_publish = NULL;
+    PyObject* result = NULL;
+    codec_options_t options;
+    buffer_t buffer;
+    struct module_state *state = GETSTATE(self);
+
+    if (!PyArg_ParseTuple(args, "bOObbO&O",
+                          &op, &command, &docs, &check_keys, &ack,
+                          convert_codec_options, &options,
+                          &ctx)) {
+        return NULL;
+    }
+    if (!(buffer = buffer_new())) {
+        PyErr_NoMemory();
+        destroy_codec_options(&options);
+        return NULL;
+    }
+    /* Save space for message length and request id */
+    if ((buffer_save_space(buffer, 8)) == -1) {
+        PyErr_NoMemory();
+        goto fail;
+    }
+    if (!buffer_write_bytes(buffer,
+                            "\x00\x00\x00\x00"  /* responseTo */
+                            "\xdd\x07\x00\x00", /* opcode */
+                            8)) {
+        goto fail;
+    }
+    if (!(to_publish = PyList_New(0))) {
+        goto fail;
+    }
+
+    if (!_batched_op_msg(
+            op,
+            check_keys,
+            ack,
+            command,
+            docs,
+            ctx,
+            to_publish,
+            options,
+            buffer,
+            state)) {
+        goto fail;
+    }
+
+    request_id = rand();
+    position = buffer_get_position(buffer);
+    buffer_write_int32_at_position(buffer, 0, (int32_t)position);
+    buffer_write_int32_at_position(buffer, 4, (int32_t)request_id);
+    result = Py_BuildValue("i" BYTES_FORMAT_STRING "O", request_id,
+                           buffer_get_buffer(buffer),
+                           buffer_get_position(buffer),
+                           to_publish);
+fail:
+    destroy_codec_options(&options);
+    buffer_free(buffer);
+    Py_XDECREF(to_publish);
+    return result;
+}
+
+/* End OP_MSG -------------------------------------------- */
+
+static int
+_batched_write_command(
+        char* ns, int ns_len, unsigned char op, int check_keys,
+        PyObject* command, PyObject* docs, PyObject* ctx,
+        PyObject* to_publish, codec_options_t options,
+        buffer_t buffer, struct module_state *state) {
+
+    long max_bson_size;
+    long max_cmd_size;
+    long max_write_batch_size;
+    int idx = 0;
+    int cmd_len_loc;
+    int lst_len_loc;
+    int position;
+    int length;
+    PyObject* max_bson_size_obj;
+    PyObject* max_write_batch_size_obj;
+    PyObject* doc;
+    PyObject* iterator;
+
+    max_bson_size_obj = PyObject_GetAttrString(ctx, "max_bson_size");
+#if PY_MAJOR_VERSION >= 3
+    max_bson_size = PyLong_AsLong(max_bson_size_obj);
+#else
+    max_bson_size = PyInt_AsLong(max_bson_size_obj);
+#endif
+    Py_XDECREF(max_bson_size_obj);
+    if (max_bson_size == -1) {
+        return 0;
+    }
     /*
      * Max BSON object size + 16k - 2 bytes for ending NUL bytes
      * XXX: This should come from the server - SERVER-10643
@@ -1067,31 +1494,26 @@ _cbson_do_batched_write_command(PyObject* self, PyObject* args) {
 #endif
     Py_XDECREF(max_write_batch_size_obj);
     if (max_write_batch_size == -1) {
-        destroy_codec_options(&options);
-        PyMem_Free(ns);
-        return NULL;
+        return 0;
     }
 
-    if (!(to_publish = PyList_New(0))) {
-        destroy_codec_options(&options);
-        PyMem_Free(ns);
-        return NULL;
+    if (!buffer_write_bytes(buffer,
+                            "\x00\x00\x00\x00", /* flags */
+                            4) ||
+        !buffer_write_bytes(buffer,
+                            ns, ns_len + 1) ||  /* namespace */
+        !buffer_write_bytes(buffer,
+                            "\x00\x00\x00\x00"  /* skip */
+                            "\xFF\xFF\xFF\xFF", /* limit (-1) */
+                             8)) {
+        return 0;
     }
 
-    if (!(buffer = _command_buffer_new(ns, ns_len))) {
-        destroy_codec_options(&options);
-        PyMem_Free(ns);
-        Py_DECREF(to_publish);
-        return NULL;
-    }
-
-    PyMem_Free(ns);
-
     /* Position of command document length */
     cmd_len_loc = buffer_get_position(buffer);
     if (!write_dict(state->_cbson, buffer, command, 0,
                     &options, 0)) {
-        goto cmdfail;
+        return 0;
     }
 
     /* Write type byte for array */
@@ -1127,7 +1549,7 @@ _cbson_do_batched_write_command(PyObject* self, PyObject* args) {
                 PyErr_SetString(InvalidOperation, "Unknown command");
                 Py_DECREF(InvalidOperation);
             }
-            goto cmdfail;
+            return 0;
         }
     }
 
@@ -1135,7 +1557,7 @@ _cbson_do_batched_write_command(PyObject* self, PyObject* args) {
     lst_len_loc = buffer_save_space(buffer, 4);
     if (lst_len_loc == -1) {
         PyErr_NoMemory();
-        goto cmdfail;
+        return 0;
     }
 
     iterator = PyObject_GetIter(docs);
@@ -1145,7 +1567,7 @@ _cbson_do_batched_write_command(PyObject* self, PyObject* args) {
             PyErr_SetString(InvalidOperation, "input is not iterable");
             Py_DECREF(InvalidOperation);
         }
-        goto cmdfail;
+        return 0;
     }
     while ((doc = PyIter_Next(iterator)) != NULL) {
         int sub_doc_begin = buffer_get_position(buffer);
@@ -1182,10 +1604,12 @@ _cbson_do_batched_write_command(PyObject* self, PyObject* args) {
                     if (DocumentTooLarge) {
                         /*
                          * There's nothing intelligent we can say
-                         * about size for update and remove.
+                         * about size for update and delete.
                          */
-                        PyErr_SetString(DocumentTooLarge,
-                                        "command document too large");
+                        PyErr_Format(
+                            DocumentTooLarge,
+                            "%s command document too large",
+                            (op == _UPDATE) ? "update": "delete");
                         Py_DECREF(DocumentTooLarge);
                     }
                 }
@@ -1214,32 +1638,151 @@ _cbson_do_batched_write_command(PyObject* self, PyObject* args) {
         goto cmdfail;
 
 
-    request_id = rand();
     position = buffer_get_position(buffer);
     length = position - lst_len_loc - 1;
     buffer_write_int32_at_position(buffer, lst_len_loc, (int32_t)length);
     length = position - cmd_len_loc;
     buffer_write_int32_at_position(buffer, cmd_len_loc, (int32_t)length);
+    return 1;
+
+cmditerfail:
+    Py_XDECREF(doc);
+    Py_DECREF(iterator);
+cmdfail:
+    return 0;
+}
+
+static PyObject*
+_cbson_encode_batched_write_command(PyObject* self, PyObject* args) {
+    char *ns = NULL;
+    unsigned char op;
+    unsigned char check_keys;
+    int ns_len;
+    PyObject* command;
+    PyObject* docs;
+    PyObject* ctx = NULL;
+    PyObject* to_publish = NULL;
+    PyObject* result = NULL;
+    codec_options_t options;
+    buffer_t buffer;
+    struct module_state *state = GETSTATE(self);
+
+    if (!PyArg_ParseTuple(args, "et#bOObO&O", "utf-8",
+                          &ns, &ns_len, &op, &command, &docs, &check_keys,
+                          convert_codec_options, &options,
+                          &ctx)) {
+        return NULL;
+    }
+    if (!(buffer = buffer_new())) {
+        PyErr_NoMemory();
+        PyMem_Free(ns);
+        destroy_codec_options(&options);
+        return NULL;
+    }
+    if (!(to_publish = PyList_New(0))) {
+        goto fail;
+    }
+
+    if (!_batched_write_command(
+            ns,
+            ns_len,
+            op,
+            check_keys,
+            command,
+            docs,
+            ctx,
+            to_publish,
+            options,
+            buffer,
+            state)) {
+        goto fail;
+    }
+
+    result = Py_BuildValue(BYTES_FORMAT_STRING "O",
+                           buffer_get_buffer(buffer),
+                           buffer_get_position(buffer),
+                           to_publish);
+fail:
+    PyMem_Free(ns);
+    destroy_codec_options(&options);
+    buffer_free(buffer);
+    Py_XDECREF(to_publish);
+    return result;
+}
+
+static PyObject*
+_cbson_batched_write_command(PyObject* self, PyObject* args) {
+    char *ns = NULL;
+    unsigned char op;
+    unsigned char check_keys;
+    int ns_len;
+    int request_id;
+    int position;
+    PyObject* command;
+    PyObject* docs;
+    PyObject* ctx = NULL;
+    PyObject* to_publish = NULL;
+    PyObject* result = NULL;
+    codec_options_t options;
+    buffer_t buffer;
+    struct module_state *state = GETSTATE(self);
+
+    if (!PyArg_ParseTuple(args, "et#bOObO&O", "utf-8",
+                          &ns, &ns_len, &op, &command, &docs, &check_keys,
+                          convert_codec_options, &options,
+                          &ctx)) {
+        return NULL;
+    }
+    if (!(buffer = buffer_new())) {
+        PyErr_NoMemory();
+        PyMem_Free(ns);
+        destroy_codec_options(&options);
+        return NULL;
+    }
+    /* Save space for message length and request id */
+    if ((buffer_save_space(buffer, 8)) == -1) {
+        PyErr_NoMemory();
+        goto fail;
+    }
+    if (!buffer_write_bytes(buffer,
+                            "\x00\x00\x00\x00"  /* responseTo */
+                            "\xd4\x07\x00\x00", /* opcode */
+                            8)) {
+        goto fail;
+    }
+    if (!(to_publish = PyList_New(0))) {
+        goto fail;
+    }
+
+    if (!_batched_write_command(
+            ns,
+            ns_len,
+            op,
+            check_keys,
+            command,
+            docs,
+            ctx,
+            to_publish,
+            options,
+            buffer,
+            state)) {
+        goto fail;
+    }
+
+    request_id = rand();
+    position = buffer_get_position(buffer);
     buffer_write_int32_at_position(buffer, 0, (int32_t)position);
     buffer_write_int32_at_position(buffer, 4, (int32_t)request_id);
     result = Py_BuildValue("i" BYTES_FORMAT_STRING "O", request_id,
                            buffer_get_buffer(buffer),
                            buffer_get_position(buffer),
                            to_publish);
-
-    Py_DECREF(to_publish);
+fail:
+    PyMem_Free(ns);
+    destroy_codec_options(&options);
     buffer_free(buffer);
-    destroy_codec_options(&options);
-    return result;
-
-cmditerfail:
-    Py_XDECREF(doc);
-    Py_DECREF(iterator);
-cmdfail:
-    destroy_codec_options(&options);
     Py_XDECREF(to_publish);
-    buffer_free(buffer);
-    return NULL;
+    return result;
 }
 
 static PyMethodDef _CMessageMethods[] = {
@@ -1251,10 +1794,18 @@ static PyMethodDef _CMessageMethods[] = {
      "create a query message to be sent to MongoDB"},
     {"_get_more_message", _cbson_get_more_message, METH_VARARGS,
      "create a get more message to be sent to MongoDB"},
+    {"_op_msg", _cbson_op_msg, METH_VARARGS,
+     "create an OP_MSG message to be sent to MongoDB"},
     {"_do_batched_insert", _cbson_do_batched_insert, METH_VARARGS,
      "insert a batch of documents, splitting the batch as needed"},
-    {"_do_batched_write_command", _cbson_do_batched_write_command, METH_VARARGS,
+    {"_batched_write_command", _cbson_batched_write_command, METH_VARARGS,
      "Create the next batched insert, update, or delete command"},
+    {"_encode_batched_write_command", _cbson_encode_batched_write_command, METH_VARARGS,
+     "Encode the next batched insert, update, or delete command"},
+    {"_batched_op_msg", _cbson_batched_op_msg, METH_VARARGS,
+     "Create the next batched insert, update, or delete using OP_MSG"},
+    {"_encode_batched_op_msg", _cbson_encode_batched_op_msg, METH_VARARGS,
+     "Encode the next batched insert, update, or delete using OP_MSG"},
     {NULL, NULL, 0, NULL}
 };
 
