@@ -1,4 +1,4 @@
-# Copyright 2009-2015 MongoDB, Inc.
+# Copyright 2009-present MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,6 +16,7 @@
 
 import copy
 import datetime
+import warnings
 
 from collections import deque
 
@@ -27,12 +28,18 @@ from bson.py3compat import (iteritems,
 from bson.son import SON
 from pymongo import helpers
 from pymongo.common import validate_boolean, validate_is_mapping
+from pymongo.collation import validate_collation_or_none
 from pymongo.errors import (AutoReconnect,
                             ConnectionFailure,
                             InvalidOperation,
                             NotMasterError,
                             OperationFailure)
-from pymongo.message import _CursorAddress, _GetMore, _Query, _convert_exception
+from pymongo.message import (_convert_exception,
+                             _CursorAddress,
+                             _GetMore,
+                             _RawBatchGetMore,
+                             _Query,
+                             _RawBatchQuery)
 from pymongo.read_preferences import ReadPreference
 
 _QUERY_OPTIONS = {
@@ -99,12 +106,17 @@ class _SocketManager:
 class Cursor(object):
     """A cursor / iterator over Mongo query results.
     """
+    _query_class = _Query
+    _getmore_class = _GetMore
 
     def __init__(self, collection, filter=None, projection=None, skip=0,
                  limit=0, no_cursor_timeout=False,
                  cursor_type=CursorType.NON_TAILABLE,
                  sort=None, allow_partial_results=False, oplog_replay=False,
-                 modifiers=None, batch_size=0, manipulate=True):
+                 modifiers=None, batch_size=0, manipulate=True,
+                 collation=None, hint=None, max_scan=None, max_time_ms=None,
+                 max=None, min=None, return_key=False, show_record_id=False,
+                 snapshot=False, comment=None, session=None):
         """Create a new cursor.
 
         Should not be called directly by application developers - see
@@ -112,7 +124,19 @@ class Cursor(object):
 
         .. mongodoc:: cursors
         """
+        # Initialize all attributes used in __del__ before possibly raising
+        # an error to avoid attribute errors during garbage collection.
         self.__id = None
+        self.__exhaust = False
+        self.__exhaust_mgr = None
+        self.__killed = False
+
+        if session:
+            self.__session = session
+            self.__explicit_session = True
+        else:
+            self.__session = None
+            self.__explicit_session = False
 
         spec = filter
         if spec is None:
@@ -130,6 +154,8 @@ class Cursor(object):
         validate_boolean("allow_partial_results", allow_partial_results)
         validate_boolean("oplog_replay", oplog_replay)
         if modifiers is not None:
+            warnings.warn("the 'modifiers' parameter is deprecated",
+                          DeprecationWarning, stacklevel=2)
             validate_is_mapping("modifiers", modifiers)
         if not isinstance(batch_size, integer_types):
             raise TypeError("batch_size must be an integer")
@@ -149,19 +175,21 @@ class Cursor(object):
         self.__batch_size = batch_size
         self.__modifiers = modifiers and modifiers.copy() or {}
         self.__ordering = sort and helpers._index_document(sort) or None
-        self.__max_scan = None
+        self.__max_scan = max_scan
         self.__explain = False
-        self.__hint = None
-        self.__comment = None
-        self.__max_time_ms = None
+        self.__comment = comment
+        self.__max_time_ms = max_time_ms
         self.__max_await_time_ms = None
-        self.__max = None
-        self.__min = None
+        self.__max = max
+        self.__min = min
         self.__manipulate = manipulate
+        self.__collation = validate_collation_or_none(collation)
+        self.__return_key = return_key
+        self.__show_record_id = show_record_id
+        self.__snapshot = snapshot
+        self.__set_hint(hint)
 
         # Exhaust cursor support
-        self.__exhaust = False
-        self.__exhaust_mgr = None
         if cursor_type == CursorType.EXHAUST:
             if self.__collection.database.client.is_mongos:
                 raise InvalidOperation('Exhaust cursors are '
@@ -181,15 +209,13 @@ class Cursor(object):
         self.__data = deque()
         self.__address = None
         self.__retrieved = 0
-        self.__killed = False
 
         self.__codec_options = collection.codec_options
-        self.__read_preference = collection.read_preference
+        # Read preference is set when the initial find is sent.
+        self.__read_preference = None
         self.__read_concern = collection.read_concern
 
         self.__query_flags = cursor_type
-        if self.__read_preference != ReadPreference.PRIMARY:
-            self.__query_flags |= _QUERY_OPTIONS["slave_okay"]
         if no_cursor_timeout:
             self.__query_flags |= _QUERY_OPTIONS["no_timeout"]
         if allow_partial_results:
@@ -211,8 +237,7 @@ class Cursor(object):
         return self.__retrieved
 
     def __del__(self):
-        if self.__id and not self.__killed:
-            self.__die()
+        self.__die()
 
     def rewind(self):
         """Rewind this cursor to its unevaluated state.
@@ -241,50 +266,62 @@ class Cursor(object):
         """
         return self._clone(True)
 
-    def _clone(self, deepcopy=True):
+    def _clone(self, deepcopy=True, base=None):
         """Internal clone helper."""
-        clone = self._clone_base()
+        if not base:
+            if self.__explicit_session:
+                base = self._clone_base(self.__session)
+            else:
+                base = self._clone_base(None)
+
         values_to_clone = ("spec", "projection", "skip", "limit",
                            "max_time_ms", "max_await_time_ms", "comment",
                            "max", "min", "ordering", "explain", "hint",
                            "batch_size", "max_scan", "manipulate",
-                           "query_flags", "modifiers")
+                           "query_flags", "modifiers", "collation")
         data = dict((k, v) for k, v in iteritems(self.__dict__)
                     if k.startswith('_Cursor__') and k[9:] in values_to_clone)
         if deepcopy:
             data = self._deepcopy(data)
-        clone.__dict__.update(data)
-        return clone
+        base.__dict__.update(data)
+        return base
 
-    def _clone_base(self):
+    def _clone_base(self, session):
         """Creates an empty Cursor object for information to be copied into.
         """
-        return Cursor(self.__collection)
+        return self.__class__(self.__collection, session=session)
 
-    def __die(self):
+    def __die(self, synchronous=False):
         """Closes this cursor.
         """
-        if self.__id and not self.__killed:
+        already_killed = self.__killed
+        self.__killed = True
+        if self.__id and not already_killed:
             if self.__exhaust and self.__exhaust_mgr:
                 # If this is an exhaust cursor and we haven't completely
                 # exhausted the result set we *must* close the socket
                 # to stop the server from sending more data.
                 self.__exhaust_mgr.sock.close()
             else:
-                self.__collection.database.client.close_cursor(
-                    self.__id,
-                    _CursorAddress(
-                        self.__address, self.__collection.full_name))
+                address = _CursorAddress(
+                    self.__address, self.__collection.full_name)
+                if synchronous:
+                    self.__collection.database.client._close_cursor_now(
+                        self.__id, address, session=self.__session)
+                else:
+                    # The cursor will be closed later in a different session.
+                    self.__collection.database.client._close_cursor(
+                        self.__id, address)
         if self.__exhaust and self.__exhaust_mgr:
             self.__exhaust_mgr.close()
-        self.__killed = True
+        if self.__session and not self.__explicit_session:
+            self.__session._end_session(lock=synchronous)
+            self.__session = None
 
     def close(self):
-        """Explicitly close / kill this cursor. Required for PyPy, Jython and
-        other Python implementations that don't use reference counting
-        garbage collection.
+        """Explicitly close / kill this cursor.
         """
-        self.__die()
+        self.__die(True)
 
     def __query_spec(self):
         """Get the spec to use for a query.
@@ -306,6 +343,13 @@ class Cursor(object):
             operators["$max"] = self.__max
         if self.__min:
             operators["$min"] = self.__min
+        if self.__return_key:
+            operators["$returnKey"] = self.__return_key
+        if self.__show_record_id:
+            # This is upgraded to showRecordId for MongoDB 3.2+ "find" command.
+            operators["$showDiskLoc"] = self.__show_record_id
+        if self.__snapshot:
+            operators["$snapshot"] = self.__snapshot
 
         if operators:
             # Make a shallow copy so we can cleanly rewind or clone.
@@ -426,7 +470,7 @@ class Cursor(object):
             raise ValueError("batch_size must be >= 0")
         self.__check_okay_to_chain()
 
-        self.__batch_size = batch_size == 1 and 2 or batch_size
+        self.__batch_size = batch_size
         return self
 
     def skip(self, skip):
@@ -537,14 +581,14 @@ class Cursor(object):
             skip = 0
             if index.start is not None:
                 if index.start < 0:
-                    raise IndexError("Cursor instances do not support"
+                    raise IndexError("Cursor instances do not support "
                                      "negative indices")
                 skip = index.start
 
             if index.stop is not None:
                 limit = index.stop - skip
                 if limit < 0:
-                    raise IndexError("stop index must be greater than start"
+                    raise IndexError("stop index must be greater than start "
                                      "index for slice %r" % index)
                 if limit == 0:
                     self.__empty = True
@@ -557,11 +601,12 @@ class Cursor(object):
 
         if isinstance(index, integer_types):
             if index < 0:
-                raise IndexError("Cursor instances do not support negative"
+                raise IndexError("Cursor instances do not support negative "
                                  "indices")
             clone = self.clone()
             clone.skip(index + self.__skip)
             clone.limit(-1)  # use a hard limit
+            clone.__query_flags &= ~CursorType.TAILABLE_AWAIT  # PYTHON-1371
             for doc in clone:
                 return doc
             raise IndexError("no such item for Cursor instance")
@@ -569,7 +614,8 @@ class Cursor(object):
                         "instances" % index)
 
     def max_scan(self, max_scan):
-        """Limit the number of documents to scan when performing the query.
+        """**DEPRECATED** - Limit the number of documents to scan when
+        performing the query.
 
         Raises :class:`~pymongo.errors.InvalidOperation` if this
         cursor has already been used. Only the last :meth:`max_scan`
@@ -577,6 +623,11 @@ class Cursor(object):
 
         :Parameters:
           - `max_scan`: the maximum number of documents to scan
+
+        .. versionchanged:: 3.7
+          Deprecated :meth:`max_scan`. Support for this option is deprecated in
+          MongoDB 4.0. Use :meth:`max_time_ms` instead to limit server side
+          execution time.
         """
         self.__check_okay_to_chain()
         self.__max_scan = max_scan
@@ -659,7 +710,11 @@ class Cursor(object):
         return self
 
     def count(self, with_limit_and_skip=False):
-        """Get the size of the results set for this query.
+        """**DEPRECATED** - Get the size of the results set for this query.
+
+        The :meth:`count` method is deprecated and **not** supported in a
+        transaction. Please use
+        :meth:`~pymongo.collection.Collection.count_documents` instead.
 
         Returns the number of documents in the results set for this query. Does
         not take :meth:`limit` and :meth:`skip` into account by default - set
@@ -685,9 +740,14 @@ class Cursor(object):
         .. note:: The `with_limit_and_skip` parameter requires server
            version **>= 1.1.4-**
 
+        .. versionchanged:: 3.7
+           Deprecated.
+
         .. versionchanged:: 2.8
            The :meth:`~count` method now supports :meth:`~hint`.
         """
+        warnings.warn("count is deprecated. Use Collection.count_documents "
+                      "instead.", DeprecationWarning, stacklevel=2)
         validate_boolean("with_limit_and_skip", with_limit_and_skip)
         cmd = SON([("count", self.__collection.name),
                    ("query", self.__spec)])
@@ -705,7 +765,8 @@ class Cursor(object):
             if self.__skip:
                 cmd["skip"] = self.__skip
 
-        return self.__collection._count(cmd)
+        return self.__collection._count(
+            cmd, self.__collation, session=self.__session)
 
     def distinct(self, key):
         """Get a list of distinct values for `key` among all documents
@@ -731,8 +792,11 @@ class Cursor(object):
             options['maxTimeMS'] = self.__max_time_ms
         if self.__comment:
             options['$comment'] = self.__comment
+        if self.__collation is not None:
+            options['collation'] = self.__collation
 
-        return self.__collection.distinct(key, **options)
+        return self.__collection.distinct(
+            key, session=self.__session, **options)
 
     def explain(self):
         """Returns an explain plan record for this cursor.
@@ -747,6 +811,16 @@ class Cursor(object):
             c.__limit = -abs(c.__limit)
         return next(c)
 
+    def __set_hint(self, index):
+        if index is None:
+            self.__hint = None
+            return
+
+        if isinstance(index, string_type):
+            self.__hint = index
+        else:
+            self.__hint = helpers._index_document(index)
+
     def hint(self, index):
         """Adds a 'hint', telling Mongo the proper index to use for the query.
 
@@ -772,14 +846,7 @@ class Cursor(object):
            The :meth:`~hint` method accepts the name of the index.
         """
         self.__check_okay_to_chain()
-        if index is None:
-            self.__hint = None
-            return self
-
-        if isinstance(index, string_type):
-            self.__hint = index
-        else:
-            self.__hint = helpers._index_document(index)
+        self.__set_hint(index)
         return self
 
     def comment(self, comment):
@@ -823,6 +890,24 @@ class Cursor(object):
         self.__spec["$where"] = code
         return self
 
+    def collation(self, collation):
+        """Adds a :class:`~pymongo.collation.Collation` to this query.
+
+        This option is only supported on MongoDB 3.4 and above.
+
+        Raises :exc:`TypeError` if `collation` is not an instance of
+        :class:`~pymongo.collation.Collation` or a ``dict``. Raises
+        :exc:`~pymongo.errors.InvalidOperation` if this :class:`Cursor` has
+        already been used. Only the last collation applied to this cursor has
+        any effect.
+
+        :Parameters:
+          - `collation`: An instance of :class:`~pymongo.collation.Collation`.
+        """
+        self.__check_okay_to_chain()
+        self.__collation = validate_collation_or_none(collation)
+        return self
+
     def __send_message(self, operation):
         """Send a query or getmore operation and handles the response.
 
@@ -836,18 +921,14 @@ class Cursor(object):
         listeners = client._event_listeners
         publish = listeners.enabled_for_commands
         from_command = False
+        start = datetime.datetime.now()
+
+        def duration(): return datetime.datetime.now() - start
 
         if operation:
-            kwargs = {
-                "read_preference": self.__read_preference,
-                "exhaust": self.__exhaust,
-            }
-            if self.__address is not None:
-                kwargs["address"] = self.__address
-
             try:
-                response = client._send_message_with_response(operation,
-                                                              **kwargs)
+                response = client._send_message_with_response(
+                    operation, exhaust=self.__exhaust, address=self.__address)
                 self.__address = response.address
                 if self.__exhaust:
                     # 'response' is an ExhaustResponse.
@@ -855,8 +936,7 @@ class Cursor(object):
                                                         response.pool)
 
                 cmd_name = operation.name
-                data = response.data
-                cmd_duration = response.duration
+                reply = response.data
                 rqst_id = response.request_id
                 from_command = response.from_command
             except AutoReconnect:
@@ -865,6 +945,7 @@ class Cursor(object):
                 # assertion on some server releases if we get here
                 # due to a socket timeout.
                 self.__killed = True
+                self.__die()
                 raise
         else:
             # Exhaust cursor - no getMore message.
@@ -880,29 +961,25 @@ class Cursor(object):
                     cmd['maxTimeMS'] = self.__max_time_ms
                 listeners.publish_command_start(
                     cmd, self.__collection.database.name, 0, self.__address)
-                start = datetime.datetime.now()
             try:
-                data = self.__exhaust_mgr.sock.receive_message(1, None)
+                reply = self.__exhaust_mgr.sock.receive_message(None)
             except Exception as exc:
                 if publish:
-                    duration = datetime.datetime.now() - start
                     listeners.publish_command_failure(
-                        duration, _convert_exception(exc), cmd_name, rqst_id,
+                        duration(), _convert_exception(exc), cmd_name, rqst_id,
                         self.__address)
                 if isinstance(exc, ConnectionFailure):
                     self.__die()
                 raise
-            if publish:
-                cmd_duration = datetime.datetime.now() - start
 
-        if publish:
-            start = datetime.datetime.now()
         try:
-            doc = helpers._unpack_response(response=data,
-                                           cursor_id=self.__id,
-                                           codec_options=self.__codec_options)
+            docs = self._unpack_response(response=reply,
+                                         cursor_id=self.__id,
+                                         codec_options=self.__codec_options)
             if from_command:
-                helpers._check_command_response(doc['data'][0])
+                first = docs[0]
+                client._receive_cluster_time(first, self.__session)
+                helpers._check_command_response(first)
         except OperationFailure as exc:
             self.__killed = True
 
@@ -910,9 +987,8 @@ class Cursor(object):
             self.__die()
 
             if publish:
-                duration = (datetime.datetime.now() - start) + cmd_duration
                 listeners.publish_command_failure(
-                    duration, exc.details, cmd_name, rqst_id, self.__address)
+                    duration(), exc.details, cmd_name, rqst_id, self.__address)
 
             # If this is a tailable cursor the error is likely
             # due to capped collection roll over. Setting
@@ -930,63 +1006,72 @@ class Cursor(object):
             self.__die()
 
             if publish:
-                duration = (datetime.datetime.now() - start) + cmd_duration
                 listeners.publish_command_failure(
-                    duration, exc.details, cmd_name, rqst_id, self.__address)
+                    duration(), exc.details, cmd_name, rqst_id, self.__address)
 
             client._reset_server_and_request_check(self.__address)
             raise
         except Exception as exc:
             if publish:
-                duration = (datetime.datetime.now() - start) + cmd_duration
                 listeners.publish_command_failure(
-                    duration, _convert_exception(exc), cmd_name, rqst_id,
+                    duration(), _convert_exception(exc), cmd_name, rqst_id,
                     self.__address)
             raise
 
         if publish:
-            duration = (datetime.datetime.now() - start) + cmd_duration
             # Must publish in find / getMore / explain command response format.
             if from_command:
-                res = doc['data'][0]
+                res = docs[0]
             elif cmd_name == "explain":
-                res = doc["data"][0] if doc["number_returned"] else {}
+                res = docs[0] if docs else {}
             else:
-                res = {"cursor": {"id": doc["cursor_id"],
+                res = {"cursor": {"id": reply.cursor_id,
                                   "ns": self.__collection.full_name},
                        "ok": 1}
                 if cmd_name == "find":
-                    res["cursor"]["firstBatch"] = doc["data"]
+                    res["cursor"]["firstBatch"] = docs
                 else:
-                    res["cursor"]["nextBatch"] = doc["data"]
+                    res["cursor"]["nextBatch"] = docs
             listeners.publish_command_success(
-                duration, res, cmd_name, rqst_id, self.__address)
+                duration(), res, cmd_name, rqst_id, self.__address)
 
-        if from_command and cmd_name != "explain":
-            cursor = doc['data'][0]['cursor']
-            self.__id = cursor['id']
-            if cmd_name == 'find':
-                documents = cursor['firstBatch']
+        if from_command:
+            if cmd_name != "explain":
+                cursor = docs[0]['cursor']
+                self.__id = cursor['id']
+                if cmd_name == 'find':
+                    documents = cursor['firstBatch']
+                else:
+                    documents = cursor['nextBatch']
+                self.__data = deque(documents)
+                self.__retrieved += len(documents)
             else:
-                documents = cursor['nextBatch']
-            self.__data = deque(documents)
-            self.__retrieved += len(documents)
+                self.__id = 0
+                self.__data = deque(docs)
+                self.__retrieved += len(docs)
         else:
-            self.__id = doc["cursor_id"]
-            self.__data = deque(doc["data"])
-            self.__retrieved += doc["number_returned"]
+            self.__id = reply.cursor_id
+            self.__data = deque(docs)
+            self.__retrieved += reply.number_returned
 
         if self.__id == 0:
             self.__killed = True
-
+            # Don't wait for garbage collection to call __del__, return the
+            # socket and the session to the pool now.
+            self.__die()
 
         if self.__limit and self.__id and self.__limit <= self.__retrieved:
             self.__die()
 
-        # Don't wait for garbage collection to call __del__, return the
-        # socket to the pool now.
-        if self.__exhaust and self.__id == 0:
-            self.__exhaust_mgr.close()
+    def _unpack_response(self, response, cursor_id, codec_options):
+        return response.unpack_response(cursor_id, codec_options)
+
+    def _read_preference(self):
+        if self.__read_preference is None:
+            # Save the read preference for getMore commands.
+            self.__read_preference = self.__collection._read_preference_for(
+                self.session)
+        return self.__read_preference
 
     def _refresh(self):
         """Refreshes the cursor with more data from Mongo.
@@ -998,27 +1083,25 @@ class Cursor(object):
         if len(self.__data) or self.__killed:
             return len(self.__data)
 
+        if not self.__session:
+            self.__session = self.__collection.database.client._ensure_session()
+
         if self.__id is None:  # Query
-            ntoreturn = self.__batch_size
-            if self.__limit:
-                if self.__batch_size:
-                    ntoreturn = min(self.__limit, self.__batch_size)
-                else:
-                    ntoreturn = self.__limit
-            self.__send_message(_Query(self.__query_flags,
-                                       self.__collection.database.name,
-                                       self.__collection.name,
-                                       self.__skip,
-                                       ntoreturn,
-                                       self.__query_spec(),
-                                       self.__projection,
-                                       self.__codec_options,
-                                       self.__read_preference,
-                                       self.__limit,
-                                       self.__batch_size,
-                                       self.__read_concern))
-            if not self.__id:
-                self.__killed = True
+            q = self._query_class(self.__query_flags,
+                                  self.__collection.database.name,
+                                  self.__collection.name,
+                                  self.__skip,
+                                  self.__query_spec(),
+                                  self.__projection,
+                                  self.__codec_options,
+                                  self._read_preference(),
+                                  self.__limit,
+                                  self.__batch_size,
+                                  self.__read_concern,
+                                  self.__collation,
+                                  self.__session,
+                                  self.__collection.database.client)
+            self.__send_message(q)
         elif self.__id:  # Get More
             if self.__limit:
                 limit = self.__limit - self.__retrieved
@@ -1031,15 +1114,16 @@ class Cursor(object):
             if self.__exhaust:
                 self.__send_message(None)
             else:
-                self.__send_message(_GetMore(self.__collection.database.name,
-                                             self.__collection.name,
-                                             limit,
-                                             self.__id,
-                                             self.__codec_options,
-                                             self.__max_await_time_ms))
-
-        else:  # Cursor id is zero nothing else to return
-            self.__killed = True
+                g = self._getmore_class(self.__collection.database.name,
+                                        self.__collection.name,
+                                        limit,
+                                        self.__id,
+                                        self.__codec_options,
+                                        self._read_preference(),
+                                        self.__session,
+                                        self.__collection.database.client,
+                                        self.__max_await_time_ms)
+                self.__send_message(g)
 
         return len(self.__data)
 
@@ -1086,6 +1170,15 @@ class Cursor(object):
         """
         return self.__address
 
+    @property
+    def session(self):
+        """The cursor's :class:`~pymongo.client_session.ClientSession`, or None.
+
+        .. versionadded:: 3.6
+        """
+        if self.__explicit_session:
+            return self.__session
+
     def __iter__(self):
         return self
 
@@ -1093,9 +1186,9 @@ class Cursor(object):
         """Advance the cursor."""
         if self.__empty:
             raise StopIteration
-        _db = self.__collection.database
         if len(self.__data) or self._refresh():
             if self.__manipulate:
+                _db = self.__collection.database
                 return _db._fix_outgoing(self.__data.popleft(),
                                          self.__collection)
             else:
@@ -1109,7 +1202,7 @@ class Cursor(object):
         return self
 
     def __exit__(self, exc_type, exc_val, exc_tb):
-        self.__die()
+        self.close()
 
     def __copy__(self):
         """Support function for `copy.copy()`.
@@ -1156,3 +1249,42 @@ class Cursor(object):
                     key = copy.deepcopy(key, memo)
                 y[key] = value
         return y
+
+
+class RawBatchCursor(Cursor):
+    """A cursor / iterator over raw batches of BSON data from a query result."""
+
+    _query_class = _RawBatchQuery
+    _getmore_class = _RawBatchGetMore
+
+    def __init__(self, *args, **kwargs):
+        """Create a new cursor / iterator over raw batches of BSON data.
+
+        Should not be called directly by application developers -
+        see :meth:`~pymongo.collection.Collection.find_raw_batches`
+        instead.
+
+        .. mongodoc:: cursors
+        """
+        manipulate = kwargs.get('manipulate')
+        kwargs['manipulate'] = False
+        super(RawBatchCursor, self).__init__(*args, **kwargs)
+
+        # Throw only after cursor's initialized, to prevent errors in __del__.
+        if manipulate:
+            raise InvalidOperation(
+                "Cannot use RawBatchCursor with manipulate=True")
+
+    def _unpack_response(self, response, cursor_id, codec_options):
+        return response.raw_response(cursor_id)
+
+    def explain(self):
+        """Returns an explain plan record for this cursor.
+
+        .. mongodoc:: explain
+        """
+        clone = self._clone(deepcopy=True, base=Cursor(self.collection))
+        return clone.explain()
+
+    def __getitem__(self, index):
+        raise InvalidOperation("Cannot call __getitem__ on RawBatchCursor")
