@@ -18,6 +18,7 @@
 import os
 import socket
 import sys
+import threading
 import time
 import warnings
 
@@ -41,7 +42,7 @@ import pymongo.errors
 
 from bson.son import SON
 from bson.py3compat import _unicode
-from pymongo import common
+from pymongo import common, message
 from pymongo.common import partition_node
 from pymongo.ssl_support import HAVE_SSL, validate_cert_reqs
 from test.version import Version
@@ -50,14 +51,12 @@ if HAVE_SSL:
     import ssl
 
 # The host and port of a single mongod or mongos, or the seed host
-# for a replica set. Hostnames retrieved from isMaster will be of
-# unicode type in Python 2, so ensure these hostnames are unicodes,
-# too. It makes tests like `test_repr` predictable.
-host = _unicode(os.environ.get("DB_IP", 'localhost'))
+# for a replica set.
+host = os.environ.get("DB_IP", 'localhost')
 port = int(os.environ.get("DB_PORT", 27017))
 
-db_user = _unicode(os.environ.get("DB_USER", "user"))
-db_pwd = _unicode(os.environ.get("DB_PASSWORD", "password"))
+db_user = os.environ.get("DB_USER", "user")
+db_pwd = os.environ.get("DB_PASSWORD", "password")
 
 CERT_PATH = os.path.join(os.path.dirname(os.path.realpath(__file__)),
                          'certificates')
@@ -74,6 +73,7 @@ if CA_PEM:
 if CERT_REQS is not None:
     _SSL_OPTIONS['ssl_cert_reqs'] = CERT_REQS
 
+COMPRESSORS = os.environ.get("COMPRESSORS")
 
 def is_server_resolvable():
     """Returns True if 'server' is resolvable."""
@@ -89,22 +89,6 @@ def is_server_resolvable():
         socket.setdefaulttimeout(socket_timeout)
 
 
-def _connect(host, port, **kwargs):
-    client = pymongo.MongoClient(host, port, **kwargs)
-    start = time.time()
-    # Jython takes a long time to connect.
-    if sys.platform.startswith('java'):
-        time_limit = 10
-    else:
-        time_limit = .5
-    while not client.nodes:
-        time.sleep(0.05)
-        if time.time() - start > time_limit:
-            return None
-
-    return client
-
-
 def _create_user(authdb, user, pwd=None, roles=None, **kwargs):
     cmd = SON([('createUser', user)])
     # X509 doesn't use a password
@@ -178,6 +162,7 @@ class ClientContext(object):
         self.nodes = set()
         self.replica_set_name = None
         self.cmd_line = None
+        self.server_status = None
         self.version = Version(-1)  # Needs to be comparable with Version
         self.auth_enabled = False
         self.test_commands_enabled = False
@@ -188,16 +173,50 @@ class ClientContext(object):
         self.ssl_cert_none = False
         self.ssl_certfile = False
         self.server_is_resolvable = is_server_resolvable()
-        self.ssl_client_options = {}
+        self.default_client_options = {}
         self.sessions_enabled = False
-        self.client = self._connect(host, port)
+        self.client = None
+        self.conn_lock = threading.Lock()
 
+        if COMPRESSORS:
+            self.default_client_options["compressors"] = COMPRESSORS
+
+    def _connect(self, host, port, **kwargs):
+        # Jython takes a long time to connect.
+        if sys.platform.startswith('java'):
+            timeout_ms = 10000
+        else:
+            timeout_ms = 5000
+        if COMPRESSORS:
+            kwargs["compressors"] = COMPRESSORS
+        client = pymongo.MongoClient(
+            host, port, serverSelectionTimeoutMS=timeout_ms, **kwargs)
+        try:
+            try:
+                client.admin.command('isMaster')  # Can we connect?
+            except pymongo.errors.OperationFailure as exc:
+                # SERVER-32063
+                self.connection_attempts.append(
+                    'connected client %r, but isMaster failed: %s' % (
+                        client, exc))
+            else:
+                self.connection_attempts.append(
+                    'successfully connected client %r' % (client,))
+            # If connected, then return client with default timeout
+            return pymongo.MongoClient(host, port, **kwargs)
+        except pymongo.errors.ConnectionFailure as exc:
+            self.connection_attempts.append(
+                'failed to connect client %r: %s' % (client, exc))
+            return None
+
+    def _init_client(self):
+        self.client = self._connect(host, port)
         if HAVE_SSL and not self.client:
             # Is MongoDB configured for SSL?
             self.client = self._connect(host, port, **_SSL_OPTIONS)
             if self.client:
                 self.ssl = True
-                self.ssl_client_options = _SSL_OPTIONS
+                self.default_client_options.update(_SSL_OPTIONS)
                 self.ssl_certfile = True
                 if _SSL_OPTIONS.get('ssl_cert_reqs') == ssl.CERT_NONE:
                     self.ssl_cert_none = True
@@ -225,16 +244,17 @@ class ClientContext(object):
                 self.client = self._connect(
                     host, port, username=db_user, password=db_pwd,
                     replicaSet=self.replica_set_name,
-                    **self.ssl_client_options)
+                    **self.default_client_options)
 
                 # May not have this if OperationFailure was raised earlier.
                 self.cmd_line = self.client.admin.command('getCmdLineOpts')
 
+            self.server_status = self.client.admin.command('serverStatus')
             self.ismaster = ismaster = self.client.admin.command('isMaster')
             self.sessions_enabled = 'logicalSessionTimeoutMinutes' in ismaster
 
             if 'setName' in ismaster:
-                self.replica_set_name = ismaster['setName']
+                self.replica_set_name = str(ismaster['setName'])
                 self.is_rs = True
                 if self.auth_enabled:
                     # It doesn't matter which member we use as the seed here.
@@ -244,13 +264,13 @@ class ClientContext(object):
                         username=db_user,
                         password=db_pwd,
                         replicaSet=self.replica_set_name,
-                        **self.ssl_client_options)
+                        **self.default_client_options)
                 else:
                     self.client = pymongo.MongoClient(
                         host,
                         port,
                         replicaSet=self.replica_set_name,
-                        **self.ssl_client_options)
+                        **self.default_client_options)
 
                 # Get the authoritative ismaster result from the primary.
                 self.ismaster = self.client.admin.command('ismaster')
@@ -281,31 +301,10 @@ class ClientContext(object):
             self.is_mongos = (self.ismaster.get('msg') == 'isdbgrid')
             self.has_ipv6 = self._server_started_with_ipv6()
 
-    def _connect(self, host, port, **kwargs):
-        # Jython takes a long time to connect.
-        if sys.platform.startswith('java'):
-            timeout_ms = 10000
-        else:
-            timeout_ms = 500
-        client = pymongo.MongoClient(
-            host, port, serverSelectionTimeoutMS=timeout_ms, **kwargs)
-        try:
-            try:
-                client.admin.command('isMaster')  # Can we connect?
-            except pymongo.errors.OperationFailure as exc:
-                # SERVER-32063
-                self.connection_attempts.append(
-                    'connected client %r, but isMaster failed: %s' % (
-                        client, exc))
-            else:
-                self.connection_attempts.append(
-                    'successfully connected client %r' % (client,))
-            # If connected, then return client with default timeout
-            return pymongo.MongoClient(host, port, **kwargs)
-        except pymongo.errors.ConnectionFailure as exc:
-            self.connection_attempts.append(
-                'failed to connect client %r: %s' % (client, exc))
-            return None
+    def init(self):
+        with self.conn_lock:
+            if not self.client and not self.connection_attempts:
+                self._init_client()
 
     def connection_attempt_info(self):
         return '\n'.join(self.connection_attempts)
@@ -314,7 +313,7 @@ class ClientContext(object):
     def host(self):
         if self.is_rs:
             primary = self.client.primary
-            return primary[0] if primary is not None else host
+            return str(primary[0]) if primary is not None else host
         return host
 
     @property
@@ -341,7 +340,7 @@ class ClientContext(object):
             username=db_user,
             password=db_pwd,
             serverSelectionTimeoutMS=100,
-            **self.ssl_client_options)
+            **self.default_client_options)
 
         try:
             return db_user in _all_users(client.admin)
@@ -396,11 +395,12 @@ class ClientContext(object):
         def make_wrapper(f):
             @wraps(f)
             def wrap(*args, **kwargs):
+                self.init()
                 # Always raise SkipTest if we can't connect to MongoDB
                 if not self.connected:
                     raise SkipTest(
                         "Cannot connect to MongoDB on %s" % (self.pair,))
-                if condition:
+                if condition():
                     return f(*args, **kwargs)
                 raise SkipTest(msg)
             return wrap
@@ -422,40 +422,59 @@ class ClientContext(object):
     def require_connection(self, func):
         """Run a test only if we can connect to MongoDB."""
         return self._require(
-            self.connected,
+            lambda: True,  # _require checks if we're connected
             "Cannot connect to MongoDB on %s" % (self.pair,),
             func=func)
 
+    def require_no_mmap(self, func):
+        """Run a test only if the server is not using the MMAPv1 storage
+        engine. Only works for standalone and replica sets; tests are
+        run regardless of storage engine on sharded clusters. """
+        def is_not_mmap():
+            if self.is_mongos:
+                return True
+            try:
+                storage_engine = self.server_status.get(
+                    'storageEngine').get('name')
+            except AttributeError:
+                # Raised if the storageEngine key does not exist or if
+                # self.server_status is None.
+                return False
+            return storage_engine != 'mmapv1'
+
+        return self._require(
+            is_not_mmap, "Storage engine must not be MMAPv1", func=func)
+
     def require_version_min(self, *ver):
         """Run a test only if the server version is at least ``version``."""
         other_version = Version(*ver)
-        return self._require(self.version >= other_version,
+        return self._require(lambda: self.version >= other_version,
                              "Server version must be at least %s"
                              % str(other_version))
 
     def require_version_max(self, *ver):
         """Run a test only if the server version is at most ``version``."""
         other_version = Version(*ver)
-        return self._require(self.version <= other_version,
+        return self._require(lambda: self.version <= other_version,
                              "Server version must be at most %s"
                              % str(other_version))
 
     def require_auth(self, func):
         """Run a test only if the server is running with auth enabled."""
         return self.check_auth_with_sharding(
-            self._require(self.auth_enabled,
+            self._require(lambda: self.auth_enabled,
                           "Authentication is not enabled on the server",
                           func=func))
 
     def require_no_auth(self, func):
         """Run a test only if the server is running without auth enabled."""
-        return self._require(not self.auth_enabled,
+        return self._require(lambda: not self.auth_enabled,
                              "Authentication must not be enabled on the server",
                              func=func)
 
     def require_replica_set(self, func):
         """Run a test only if the client is connected to a replica set."""
-        return self._require(self.is_rs,
+        return self._require(lambda: self.is_rs,
                              "Not connected to a replica set",
                              func=func)
 
@@ -463,99 +482,130 @@ class ClientContext(object):
         """Run a test only if the client is connected to a replica set that has
         `count` secondaries.
         """
-        sec_count = 0 if not self.client else len(self.client.secondaries)
-        return self._require(sec_count >= count,
-                             "Need %d secondaries, %d available"
-                             % (count, sec_count))
+        def sec_count():
+            return 0 if not self.client else len(self.client.secondaries)
+        return self._require(lambda: sec_count() >= count,
+                             "Not enough secondaries available")
 
     def require_no_replica_set(self, func):
         """Run a test if the client is *not* connected to a replica set."""
         return self._require(
-            not self.is_rs,
+            lambda: not self.is_rs,
             "Connected to a replica set, not a standalone mongod",
             func=func)
 
     def require_ipv6(self, func):
         """Run a test only if the client can connect to a server via IPv6."""
-        return self._require(self.has_ipv6,
+        return self._require(lambda: self.has_ipv6,
                              "No IPv6",
                              func=func)
 
     def require_no_mongos(self, func):
         """Run a test only if the client is not connected to a mongos."""
-        return self._require(not self.is_mongos,
+        return self._require(lambda: not self.is_mongos,
                              "Must be connected to a mongod, not a mongos",
                              func=func)
 
     def require_mongos(self, func):
         """Run a test only if the client is connected to a mongos."""
-        return self._require(self.is_mongos,
+        return self._require(lambda: self.is_mongos,
                              "Must be connected to a mongos",
                              func=func)
 
     def require_standalone(self, func):
         """Run a test only if the client is connected to a standalone."""
-        return self._require(not (self.is_mongos or self.is_rs),
+        return self._require(lambda: not (self.is_mongos or self.is_rs),
                              "Must be connected to a standalone",
                              func=func)
 
     def require_no_standalone(self, func):
         """Run a test only if the client is not connected to a standalone."""
-        return self._require(self.is_mongos or self.is_rs,
+        return self._require(lambda: self.is_mongos or self.is_rs,
                              "Must be connected to a replica set or mongos",
                              func=func)
 
     def check_auth_with_sharding(self, func):
         """Skip a test when connected to mongos < 2.0 and running with auth."""
-        condition = not (self.auth_enabled and
+        condition = lambda: not (self.auth_enabled and
                          self.is_mongos and self.version < (2,))
         return self._require(condition,
                              "Auth with sharding requires MongoDB >= 2.0.0",
                              func=func)
 
+    def require_cluster_type(self, topologies=[]):
+        """Run a test only if the client is connected to a cluster that
+        conforms to one of the specified topologies. Acceptable topologies
+        are 'single', 'replicaset', and 'sharded'."""
+        def _is_valid_topology():
+            if 'single' in topologies and not (self.is_mongos or self.is_rs):
+                return True
+            if 'replicaset' in topologies and self.is_rs:
+                return True
+            if 'sharded' in topologies and self.is_mongos:
+                return True
+            return False
+        return self._require(
+            _is_valid_topology, 
+            "Cluster type not in %s" % (topologies))
+
     def require_test_commands(self, func):
         """Run a test only if the server has test commands enabled."""
-        return self._require(self.test_commands_enabled,
+        return self._require(lambda: self.test_commands_enabled,
                              "Test commands must be enabled",
                              func=func)
 
     def require_ssl(self, func):
         """Run a test only if the client can connect over SSL."""
-        return self._require(self.ssl,
+        return self._require(lambda: self.ssl,
                              "Must be able to connect via SSL",
                              func=func)
 
     def require_no_ssl(self, func):
         """Run a test only if the client can connect over SSL."""
-        return self._require(not self.ssl,
+        return self._require(lambda: not self.ssl,
                              "Must be able to connect without SSL",
                              func=func)
 
     def require_ssl_cert_none(self, func):
         """Run a test only if the client can connect with ssl.CERT_NONE."""
-        return self._require(self.ssl_cert_none,
+        return self._require(lambda: self.ssl_cert_none,
                              "Must be able to connect with ssl.CERT_NONE",
                              func=func)
 
     def require_ssl_certfile(self, func):
         """Run a test only if the client can connect with ssl_certfile."""
-        return self._require(self.ssl_certfile,
+        return self._require(lambda: self.ssl_certfile,
                              "Must be able to connect with ssl_certfile",
                              func=func)
 
     def require_server_resolvable(self, func):
         """Run a test only if the hostname 'server' is resolvable."""
-        return self._require(self.server_is_resolvable,
+        return self._require(lambda: self.server_is_resolvable,
                              "No hosts entry for 'server'. Cannot validate "
                              "hostname in the certificate",
                              func=func)
 
     def require_sessions(self, func):
         """Run a test only if the deployment supports sessions."""
-        return self._require(self.sessions_enabled,
+        return self._require(lambda: self.sessions_enabled,
                              "Sessions not supported",
                              func=func)
 
+    def require_transactions(self, func):
+        """Run a test only if the deployment might support transactions.
+
+        *Might* because this does not test the storage engine or FCV.
+        """
+        new_func = self.require_version_min(4, 0, 0, -1)(func)
+        return self.require_replica_set(new_func)
+
+    @property
+    def supports_reindex(self):
+        """Does the connected server support reindex?"""
+        return not (
+            self.version.at_least(4, 1, 0) and
+            self.is_mongos)
+
 
 # Reusable client context
 client_context = ClientContext()
@@ -564,7 +614,18 @@ client_context = ClientContext()
 def sanitize_cmd(cmd):
     cp = cmd.copy()
     cp.pop('$clusterTime', None)
+    cp.pop('$db', None)
+    cp.pop('$readPreference', None)
     cp.pop('lsid', None)
+    # OP_MSG encoding may move the payload type one field to the
+    # end of the command. Do the same here.
+    name = next(iter(cp))
+    try:
+        identifier = message._FIELD_MAP[name]
+        docs = cp.pop(identifier)
+        cp[identifier] = docs
+    except KeyError:
+        pass
     return cp
 
 
@@ -577,10 +638,10 @@ def sanitize_reply(reply):
 
 class PyMongoTestCase(unittest.TestCase):
     def assertEqualCommand(self, expected, actual, msg=None):
-        self.assertEqual(expected, sanitize_cmd(actual), msg)
+        self.assertEqual(sanitize_cmd(expected), sanitize_cmd(actual), msg)
 
     def assertEqualReply(self, expected, actual, msg=None):
-        self.assertEqual(expected, sanitize_reply(actual), msg)
+        self.assertEqual(sanitize_reply(expected), sanitize_reply(actual), msg)
 
 
 class IntegrationTest(PyMongoTestCase):
@@ -627,6 +688,7 @@ class MockClientTest(unittest.TestCase):
 
 
 def setup():
+    client_context.init()
     warnings.resetwarnings()
     warnings.simplefilter("always")
 
