@@ -17,18 +17,17 @@
 import functools
 import warnings
 
-from test import unittest, client_context
-from test.utils import EventListener, rs_or_single_client
-
 from pymongo import monitoring
 from pymongo.collation import (
     Collation,
     CollationCaseFirst, CollationStrength, CollationAlternate,
     CollationMaxVariable)
 from pymongo.errors import ConfigurationError
-from pymongo.write_concern import WriteConcern
 from pymongo.operations import (DeleteMany, DeleteOne, IndexModel, ReplaceOne,
                                 UpdateMany, UpdateOne)
+from pymongo.write_concern import WriteConcern
+from test import unittest, client_context
+from test.utils import EventListener, ignore_deprecations, rs_or_single_client
 
 
 class TestCollationObject(unittest.TestCase):
@@ -161,6 +160,7 @@ class TestCollation(unittest.TestCase):
         self.assertCollationInLastCommand()
 
     @raisesConfigurationErrorForOldMongoDB
+    @ignore_deprecations
     def test_count(self):
         self.db.test.count(collation=self.collation)
         self.assertCollationInLastCommand()
@@ -169,6 +169,11 @@ class TestCollation(unittest.TestCase):
         self.db.test.find(collation=self.collation).count()
         self.assertCollationInLastCommand()
 
+    @raisesConfigurationErrorForOldMongoDB
+    def test_count_documents(self):
+        self.db.test.count_documents({}, collation=self.collation)
+        self.assertCollationInLastCommand()
+
     @raisesConfigurationErrorForOldMongoDB
     def test_distinct(self):
         self.db.test.distinct('foo', collation=self.collation)
@@ -195,6 +200,7 @@ class TestCollation(unittest.TestCase):
             self.last_command_started()['explain']['collation'])
 
     @raisesConfigurationErrorForOldMongoDB
+    @client_context.require_version_max(4, 1, 0, -1)
     def test_group(self):
         self.db.test.group('foo', {'foo': {'$gt': 42}}, {},
                            'function(a, b) { return a; }',
