@@ -61,12 +61,12 @@ from pymongo.results import (InsertOneResult,
                              UpdateResult,
                              DeleteResult)
 from pymongo.write_concern import WriteConcern
+from test import client_context, unittest
 from test.test_client import IntegrationTest
-from test.utils import (is_mongos, get_pool,
+from test.utils import (get_pool, ignore_deprecations, is_mongos,
                         rs_or_single_client, single_client,
                         wait_until, EventListener,
                         IMPOSSIBLE_WRITE_CONCERN)
-from test import client_context, unittest
 
 
 class TestCollectionNoConnect(unittest.TestCase):
@@ -109,6 +109,14 @@ class TestCollectionNoConnect(unittest.TestCase):
         self.assertIn("has no attribute '_does_not_exist'",
                       str(context.exception))
 
+        coll2 = coll.with_options(write_concern=WriteConcern(w=0))
+        self.assertEqual(coll2.write_concern, WriteConcern(w=0))
+        self.assertNotEqual(coll.write_concern, coll2.write_concern)
+        coll3 = coll2.subcoll
+        self.assertEqual(coll2.write_concern, coll3.write_concern)
+        coll4 = coll2["subcoll"]
+        self.assertEqual(coll2.write_concern, coll4.write_concern)
+
     def test_iteration(self):
         self.assertRaises(TypeError, next, self.db)
 
@@ -147,11 +155,13 @@ class TestCollection(IntegrationTest):
         # No Exception.
         db = client_context.client.pymongo_test
         db.create_test_no_wc.drop()
-        wait_until(lambda: 'create_test_no_wc' not in db.collection_names(),
-                 'drop create_test_no_wc collection')
+        wait_until(
+            lambda: 'create_test_no_wc' not in db.list_collection_names(),
+            'drop create_test_no_wc collection')
         Collection(db, name='create_test_no_wc', create=True)
-        wait_until(lambda: 'create_test_no_wc' in db.collection_names(),
-                 'create create_test_no_wc collection')
+        wait_until(
+            lambda: 'create_test_no_wc' in db.list_collection_names(),
+            'create create_test_no_wc collection')
         # SERVER-33317
         if (not client_context.is_mongos or not
                 client_context.version.at_least(3, 7, 0)):
@@ -163,7 +173,7 @@ class TestCollection(IntegrationTest):
 
     def test_drop_nonexistent_collection(self):
         self.db.drop_collection('test')
-        self.assertFalse('test' in self.db.collection_names())
+        self.assertFalse('test' in self.db.list_collection_names())
 
         # No exception
         self.db.drop_collection('test')
@@ -322,6 +332,9 @@ class TestCollection(IntegrationTest):
                                       mode="off")
 
     def test_reindex(self):
+        if not client_context.supports_reindex:
+            raise unittest.SkipTest(
+                "reindex is no longer supported by mongos 4.1+")
         db = self.db
         db.drop_collection("test")
         db.test.insert_one({"foo": "bar", "who": "what", "when": "how"})
@@ -547,7 +560,7 @@ class TestCollection(IntegrationTest):
         db.test.create_index([('i', ASCENDING)], unique=True, dropDups=True)
 
         # Duplicate was dropped
-        self.assertEqual(3, db.test.count())
+        self.assertEqual(3, db.test.count_documents({}))
 
         # Index was created, plus the index on _id
         self.assertEqual(2, len(db.test.index_information()))
@@ -567,7 +580,7 @@ class TestCollection(IntegrationTest):
         self.assertRaises(DuplicateKeyError, test_create)
 
         # Duplicate wasn't dropped
-        self.assertEqual(4, db.test.count())
+        self.assertEqual(4, db.test.count_documents({}))
 
         # Index wasn't created, only the default index on _id
         self.assertEqual(1, len(db.test.index_information()))
@@ -724,7 +737,7 @@ class TestCollection(IntegrationTest):
         self.assertEqual(document["_id"], result.inserted_id)
         self.assertTrue(result.acknowledged)
         self.assertIsNotNone(db.test.find_one({"_id": document["_id"]}))
-        self.assertEqual(1, db.test.count())
+        self.assertEqual(1, db.test.count_documents({}))
 
         document = {"foo": "bar"}
         result = db.test.insert_one(document)
@@ -733,7 +746,7 @@ class TestCollection(IntegrationTest):
         self.assertEqual(document["_id"], result.inserted_id)
         self.assertTrue(result.acknowledged)
         self.assertIsNotNone(db.test.find_one({"_id": document["_id"]}))
-        self.assertEqual(2, db.test.count())
+        self.assertEqual(2, db.test.count_documents({}))
 
         db = db.client.get_database(db.name,
                                     write_concern=WriteConcern(w=0))
@@ -743,7 +756,8 @@ class TestCollection(IntegrationTest):
         self.assertEqual(document["_id"], result.inserted_id)
         self.assertFalse(result.acknowledged)
         # The insert failed duplicate key...
-        wait_until(lambda: 2 == db.test.count(), 'forcing duplicate key error')
+        wait_until(lambda: 2 == db.test.count_documents({}),
+                   'forcing duplicate key error')
 
         document = RawBSONDocument(
             bson.BSON.encode({'_id': ObjectId(), 'foo': 'bar'}))
@@ -764,7 +778,7 @@ class TestCollection(IntegrationTest):
             _id = doc["_id"]
             self.assertTrue(isinstance(_id, ObjectId))
             self.assertTrue(_id in result.inserted_ids)
-            self.assertEqual(1, db.test.count({'_id': _id}))
+            self.assertEqual(1, db.test.count_documents({'_id': _id}))
         self.assertTrue(result.acknowledged)
 
         docs = [{"_id": i} for i in range(5)]
@@ -776,7 +790,7 @@ class TestCollection(IntegrationTest):
             _id = doc["_id"]
             self.assertTrue(isinstance(_id, int))
             self.assertTrue(_id in result.inserted_ids)
-            self.assertEqual(1, db.test.count({"_id": _id}))
+            self.assertEqual(1, db.test.count_documents({"_id": _id}))
         self.assertTrue(result.acknowledged)
 
         docs = [RawBSONDocument(bson.BSON.encode({"_id": i + 5}))
@@ -792,7 +806,7 @@ class TestCollection(IntegrationTest):
         result = db.test.insert_many(docs)
         self.assertTrue(isinstance(result, InsertManyResult))
         self.assertFalse(result.acknowledged)
-        self.assertEqual(20, db.test.count())
+        self.assertEqual(20, db.test.count_documents({}))
 
     def test_delete_one(self):
         self.db.test.drop()
@@ -805,13 +819,13 @@ class TestCollection(IntegrationTest):
         self.assertTrue(isinstance(result, DeleteResult))
         self.assertEqual(1, result.deleted_count)
         self.assertTrue(result.acknowledged)
-        self.assertEqual(2, self.db.test.count())
+        self.assertEqual(2, self.db.test.count_documents({}))
 
         result = self.db.test.delete_one({"y": 1})
         self.assertTrue(isinstance(result, DeleteResult))
         self.assertEqual(1, result.deleted_count)
         self.assertTrue(result.acknowledged)
-        self.assertEqual(1, self.db.test.count())
+        self.assertEqual(1, self.db.test.count_documents({}))
 
         db = self.db.client.get_database(self.db.name,
                                          write_concern=WriteConcern(w=0))
@@ -819,7 +833,7 @@ class TestCollection(IntegrationTest):
         self.assertTrue(isinstance(result, DeleteResult))
         self.assertRaises(InvalidOperation, lambda: result.deleted_count)
         self.assertFalse(result.acknowledged)
-        wait_until(lambda: 0 == db.test.count(), 'delete 1 documents')
+        wait_until(lambda: 0 == db.test.count_documents({}), 'delete 1 documents')
 
     def test_delete_many(self):
         self.db.test.drop()
@@ -833,7 +847,7 @@ class TestCollection(IntegrationTest):
         self.assertTrue(isinstance(result, DeleteResult))
         self.assertEqual(2, result.deleted_count)
         self.assertTrue(result.acknowledged)
-        self.assertEqual(0, self.db.test.count({"x": 1}))
+        self.assertEqual(0, self.db.test.count_documents({"x": 1}))
 
         db = self.db.client.get_database(self.db.name,
                                          write_concern=WriteConcern(w=0))
@@ -841,7 +855,8 @@ class TestCollection(IntegrationTest):
         self.assertTrue(isinstance(result, DeleteResult))
         self.assertRaises(InvalidOperation, lambda: result.deleted_count)
         self.assertFalse(result.acknowledged)
-        wait_until(lambda: 0 == db.test.count(), 'delete 2 documents')
+        wait_until(
+            lambda: 0 == db.test.count_documents({}), 'delete 2 documents')
 
     def test_command_document_too_large(self):
         large = '*' * (self.client.max_bson_size + _COMMAND_OVERHEAD)
@@ -873,8 +888,14 @@ class TestCollection(IntegrationTest):
         self.assertTrue(isinstance(result, InsertOneResult))
         self.assertEqual(2, result.inserted_id)
 
-        self.assertRaises(OperationFailure, db_w0.test.insert_one,
-                          {"x": 1}, bypass_document_validation=True)
+        if client_context.version < (3, 6):
+            # Uses OP_INSERT which does not support bypass_document_validation.
+            self.assertRaises(OperationFailure, db_w0.test.insert_one,
+                              {"y": 1}, bypass_document_validation=True)
+        else:
+            db_w0.test.insert_one({"y": 1}, bypass_document_validation=True)
+            wait_until(lambda: db_w0.test.find_one({"y": 1}),
+                       "find w:0 inserted document")
 
         # Test insert_many
         docs = [{"_id": i, "x": 100 - i} for i in range(3, 100)]
@@ -886,7 +907,7 @@ class TestCollection(IntegrationTest):
             _id = doc["_id"]
             self.assertTrue(isinstance(_id, int))
             self.assertTrue(_id in result.inserted_ids)
-            self.assertEqual(1, db.test.count({"x": doc["x"]}))
+            self.assertEqual(1, db.test.count_documents({"x": doc["x"]}))
         self.assertTrue(result.acknowledged)
         docs = [{"_id": i, "a": 200 - i} for i in range(100, 200)]
         result = db.test.insert_many(docs)
@@ -896,7 +917,7 @@ class TestCollection(IntegrationTest):
             _id = doc["_id"]
             self.assertTrue(isinstance(_id, int))
             self.assertTrue(_id in result.inserted_ids)
-            self.assertEqual(1, db.test.count({"a": doc["a"]}))
+            self.assertEqual(1, db.test.count_documents({"a": doc["a"]}))
         self.assertTrue(result.acknowledged)
 
         self.assertRaises(OperationFailure, db_w0.test.insert_many,
@@ -915,33 +936,42 @@ class TestCollection(IntegrationTest):
         db.test.insert_one({"a": 101})
         self.assertRaises(OperationFailure, db.test.replace_one,
                           {"a": 101}, {"y": 1})
-        self.assertEqual(0, db.test.count({"y": 1}))
-        self.assertEqual(1, db.test.count({"a": 101}))
+        self.assertEqual(0, db.test.count_documents({"y": 1}))
+        self.assertEqual(1, db.test.count_documents({"a": 101}))
         db.test.replace_one({"a": 101}, {"y": 1},
                             bypass_document_validation=True)
-        self.assertEqual(0, db.test.count({"a": 101}))
-        self.assertEqual(1, db.test.count({"y": 1}))
+        self.assertEqual(0, db.test.count_documents({"a": 101}))
+        self.assertEqual(1, db.test.count_documents({"y": 1}))
         db.test.replace_one({"y": 1}, {"a": 102})
-        self.assertEqual(0, db.test.count({"y": 1}))
-        self.assertEqual(0, db.test.count({"a": 101}))
-        self.assertEqual(1, db.test.count({"a": 102}))
+        self.assertEqual(0, db.test.count_documents({"y": 1}))
+        self.assertEqual(0, db.test.count_documents({"a": 101}))
+        self.assertEqual(1, db.test.count_documents({"a": 102}))
 
         db.test.insert_one({"y": 1}, bypass_document_validation=True)
         self.assertRaises(OperationFailure, db.test.replace_one,
                           {"y": 1}, {"x": 101})
-        self.assertEqual(0, db.test.count({"x": 101}))
-        self.assertEqual(1, db.test.count({"y": 1}))
+        self.assertEqual(0, db.test.count_documents({"x": 101}))
+        self.assertEqual(1, db.test.count_documents({"y": 1}))
         db.test.replace_one({"y": 1}, {"x": 101},
                             bypass_document_validation=True)
-        self.assertEqual(0, db.test.count({"y": 1}))
-        self.assertEqual(1, db.test.count({"x": 101}))
+        self.assertEqual(0, db.test.count_documents({"y": 1}))
+        self.assertEqual(1, db.test.count_documents({"x": 101}))
         db.test.replace_one({"x": 101}, {"a": 103},
                             bypass_document_validation=False)
-        self.assertEqual(0, db.test.count({"x": 101}))
-        self.assertEqual(1, db.test.count({"a": 103}))
+        self.assertEqual(0, db.test.count_documents({"x": 101}))
+        self.assertEqual(1, db.test.count_documents({"a": 103}))
 
-        self.assertRaises(OperationFailure, db_w0.test.replace_one, {"y": 1},
-                          {"x": 1}, bypass_document_validation=True)
+        db.test.insert_one({"y": 1}, bypass_document_validation=True)
+        if client_context.version < (3, 6):
+            # Uses OP_UPDATE which does not support bypass_document_validation.
+            self.assertRaises(OperationFailure, db_w0.test.replace_one,
+                              {"y": 1}, {"x": 1},
+                              bypass_document_validation=True)
+        else:
+            db_w0.test.replace_one({"y": 1}, {"x": 1},
+                                   bypass_document_validation=True)
+            wait_until(lambda: db_w0.test.find_one({"x": 1}),
+                       "find w:0 replaced document")
 
     @client_context.require_version_min(3, 1, 9, -1)
     def test_update_bypass_document_validation(self):
@@ -956,34 +986,43 @@ class TestCollection(IntegrationTest):
         # Test update_one
         self.assertRaises(OperationFailure, db.test.update_one,
                           {"z": 5}, {"$inc": {"z": -10}})
-        self.assertEqual(0, db.test.count({"z": -5}))
-        self.assertEqual(1, db.test.count({"z": 5}))
+        self.assertEqual(0, db.test.count_documents({"z": -5}))
+        self.assertEqual(1, db.test.count_documents({"z": 5}))
         db.test.update_one({"z": 5}, {"$inc": {"z": -10}},
                            bypass_document_validation=True)
-        self.assertEqual(0, db.test.count({"z": 5}))
-        self.assertEqual(1, db.test.count({"z": -5}))
+        self.assertEqual(0, db.test.count_documents({"z": 5}))
+        self.assertEqual(1, db.test.count_documents({"z": -5}))
         db.test.update_one({"z": -5}, {"$inc": {"z": 6}},
                            bypass_document_validation=False)
-        self.assertEqual(1, db.test.count({"z": 1}))
-        self.assertEqual(0, db.test.count({"z": -5}))
+        self.assertEqual(1, db.test.count_documents({"z": 1}))
+        self.assertEqual(0, db.test.count_documents({"z": -5}))
 
         db.test.insert_one({"z": -10},
                            bypass_document_validation=True)
         self.assertRaises(OperationFailure, db.test.update_one,
                           {"z": -10}, {"$inc": {"z": 1}})
-        self.assertEqual(0, db.test.count({"z": -9}))
-        self.assertEqual(1, db.test.count({"z": -10}))
+        self.assertEqual(0, db.test.count_documents({"z": -9}))
+        self.assertEqual(1, db.test.count_documents({"z": -10}))
         db.test.update_one({"z": -10}, {"$inc": {"z": 1}},
                            bypass_document_validation=True)
-        self.assertEqual(1, db.test.count({"z": -9}))
-        self.assertEqual(0, db.test.count({"z": -10}))
+        self.assertEqual(1, db.test.count_documents({"z": -9}))
+        self.assertEqual(0, db.test.count_documents({"z": -10}))
         db.test.update_one({"z": -9}, {"$inc": {"z": 9}},
                            bypass_document_validation=False)
-        self.assertEqual(0, db.test.count({"z": -9}))
-        self.assertEqual(1, db.test.count({"z": 0}))
+        self.assertEqual(0, db.test.count_documents({"z": -9}))
+        self.assertEqual(1, db.test.count_documents({"z": 0}))
 
-        self.assertRaises(OperationFailure, db_w0.test.update_one, {"y": 1},
-                          {"$inc": {"x": 1}}, bypass_document_validation=True)
+        db.test.insert_one({"y": 1, "x": 0}, bypass_document_validation=True)
+        if client_context.version < (3, 6):
+            # Uses OP_UPDATE which does not support bypass_document_validation.
+            self.assertRaises(OperationFailure, db_w0.test.update_one,
+                              {"y": 1}, {"$inc": {"x": 1}},
+                              bypass_document_validation=True)
+        else:
+            db_w0.test.update_one({"y": 1}, {"$inc": {"x": 1}},
+                                  bypass_document_validation=True)
+            wait_until(lambda: db_w0.test.find_one({"y": 1, "x": 1}),
+                       "find w:0 updated document")
 
         # Test update_many
         db.test.insert_many([{"z": i} for i in range(3, 101)])
@@ -991,35 +1030,46 @@ class TestCollection(IntegrationTest):
                            bypass_document_validation=True)
         self.assertRaises(OperationFailure, db.test.update_many, {},
                           {"$inc": {"z": -100}})
-        self.assertEqual(100, db.test.count({"z": {"$gte": 0}}))
-        self.assertEqual(0, db.test.count({"z": {"$lt": 0}}))
-        self.assertEqual(0, db.test.count({"y": 0, "z": -100}))
+        self.assertEqual(100, db.test.count_documents({"z": {"$gte": 0}}))
+        self.assertEqual(0, db.test.count_documents({"z": {"$lt": 0}}))
+        self.assertEqual(0, db.test.count_documents({"y": 0, "z": -100}))
         db.test.update_many({"z": {"$gte": 0}}, {"$inc": {"z": -100}},
                             bypass_document_validation=True)
-        self.assertEqual(0, db.test.count({"z": {"$gt": 0}}))
-        self.assertEqual(100, db.test.count({"z": {"$lte": 0}}))
+        self.assertEqual(0, db.test.count_documents({"z": {"$gt": 0}}))
+        self.assertEqual(100, db.test.count_documents({"z": {"$lte": 0}}))
         db.test.update_many({"z": {"$gt": -50}}, {"$inc": {"z": 100}},
                             bypass_document_validation=False)
-        self.assertEqual(50, db.test.count({"z": {"$gt": 0}}))
-        self.assertEqual(50, db.test.count({"z": {"$lt": 0}}))
+        self.assertEqual(50, db.test.count_documents({"z": {"$gt": 0}}))
+        self.assertEqual(50, db.test.count_documents({"z": {"$lt": 0}}))
 
         db.test.insert_many([{"z": -i} for i in range(50)],
                             bypass_document_validation=True)
         self.assertRaises(OperationFailure, db.test.update_many,
             {}, {"$inc": {"z": 1}})
-        self.assertEqual(100, db.test.count({"z": {"$lte": 0}}))
-        self.assertEqual(50, db.test.count({"z": {"$gt": 1}}))
+        self.assertEqual(100, db.test.count_documents({"z": {"$lte": 0}}))
+        self.assertEqual(50, db.test.count_documents({"z": {"$gt": 1}}))
         db.test.update_many({"z": {"$gte": 0}}, {"$inc": {"z": -100}},
                             bypass_document_validation=True)
-        self.assertEqual(0, db.test.count({"z": {"$gt": 0}}))
-        self.assertEqual(150, db.test.count({"z": {"$lte": 0}}))
+        self.assertEqual(0, db.test.count_documents({"z": {"$gt": 0}}))
+        self.assertEqual(150, db.test.count_documents({"z": {"$lte": 0}}))
         db.test.update_many({"z": {"$lte": 0}}, {"$inc": {"z": 100}},
                             bypass_document_validation=False)
-        self.assertEqual(150, db.test.count({"z": {"$gte": 0}}))
-        self.assertEqual(0, db.test.count({"z": {"$lt": 0}}))
+        self.assertEqual(150, db.test.count_documents({"z": {"$gte": 0}}))
+        self.assertEqual(0, db.test.count_documents({"z": {"$lt": 0}}))
 
-        self.assertRaises(OperationFailure, db_w0.test.update_many, {"y": 1},
-                          {"$inc": {"x": 1}}, bypass_document_validation=True)
+        db.test.insert_one({"m": 1, "x": 0}, bypass_document_validation=True)
+        db.test.insert_one({"m": 1, "x": 0}, bypass_document_validation=True)
+        if client_context.version < (3, 6):
+            # Uses OP_UPDATE which does not support bypass_document_validation.
+            self.assertRaises(OperationFailure, db_w0.test.update_many,
+                              {"m": 1}, {"$inc": {"x": 1}},
+                              bypass_document_validation=True)
+        else:
+            db_w0.test.update_many({"m": 1}, {"$inc": {"x": 1}},
+                                   bypass_document_validation=True)
+            wait_until(
+                lambda: db_w0.test.count_documents({"m": 1, "x": 1}) == 2,
+                "find w:0 updated documents")
 
     @client_context.require_version_min(3, 1, 9, -1)
     def test_bypass_document_validation_bulk_write(self):
@@ -1037,10 +1087,10 @@ class TestCollection(IntegrationTest):
                ReplaceOne({"a": {"$lte": -10}}, {"a": -1})]
         db.test.bulk_write(ops, bypass_document_validation=True)
 
-        self.assertEqual(3, db.test.count())
-        self.assertEqual(1, db.test.count({"a": -11}))
-        self.assertEqual(1, db.test.count({"a": -1}))
-        self.assertEqual(1, db.test.count({"a": -9}))
+        self.assertEqual(3, db.test.count_documents({}))
+        self.assertEqual(1, db.test.count_documents({"a": -11}))
+        self.assertEqual(1, db.test.count_documents({"a": -1}))
+        self.assertEqual(1, db.test.count_documents({"a": -9}))
 
         # Assert that the operations would fail without bypass_doc_val
         for op in ops:
@@ -1062,7 +1112,7 @@ class TestCollection(IntegrationTest):
 
         db.test.insert_one({"x": 1, "mike": "awesome",
                             "extra thing": "abcdefghijklmnopqrstuvwxyz"})
-        self.assertEqual(1, db.test.count())
+        self.assertEqual(1, db.test.count_documents({}))
         doc = next(db.test.find({}))
         self.assertTrue("x" in doc)
         doc = next(db.test.find({}))
@@ -1104,15 +1154,15 @@ class TestCollection(IntegrationTest):
         db.test.insert_one({"x": "hello_mikey"})
         db.test.insert_one({"x": "hello_test"})
 
-        self.assertEqual(db.test.find().count(), 4)
-        self.assertEqual(db.test.find({"x":
-                                       re.compile("^hello.*")}).count(), 4)
-        self.assertEqual(db.test.find({"x":
-                                       re.compile("ello")}).count(), 4)
-        self.assertEqual(db.test.find({"x":
-                                       re.compile("^hello$")}).count(), 0)
-        self.assertEqual(db.test.find({"x":
-                                       re.compile("^hello_mi.*$")}).count(), 2)
+        self.assertEqual(len(list(db.test.find())), 4)
+        self.assertEqual(len(list(db.test.find({"x":
+                                       re.compile("^hello.*")}))), 4)
+        self.assertEqual(len(list(db.test.find({"x":
+                                       re.compile("ello")}))), 4)
+        self.assertEqual(len(list(db.test.find({"x":
+                                       re.compile("^hello$")}))), 0)
+        self.assertEqual(len(list(db.test.find({"x":
+                                       re.compile("^hello_mi.*$")}))), 2)
 
     def test_id_can_be_anything(self):
         db = self.db
@@ -1195,7 +1245,7 @@ class TestCollection(IntegrationTest):
             db.test.insert_one({"x": 1})
 
         self.assertIsNotNone(context.exception.details)
-        self.assertEqual(1, db.test.count())
+        self.assertEqual(1, db.test.count_documents({}))
 
     def test_write_error_text_handling(self):
         db = self.db
@@ -1323,8 +1373,8 @@ class TestCollection(IntegrationTest):
         self.assertTrue(result.modified_count in (None, 1))
         self.assertIsNone(result.upserted_id)
         self.assertTrue(result.acknowledged)
-        self.assertEqual(1, db.test.count({"y": 1}))
-        self.assertEqual(0, db.test.count({"x": 1}))
+        self.assertEqual(1, db.test.count_documents({"y": 1}))
+        self.assertEqual(0, db.test.count_documents({"x": 1}))
         self.assertEqual(db.test.find_one(id1)["y"], 1)
 
         replacement = RawBSONDocument(bson.BSON.encode({"_id": id1, "z": 1}))
@@ -1334,8 +1384,8 @@ class TestCollection(IntegrationTest):
         self.assertTrue(result.modified_count in (None, 1))
         self.assertIsNone(result.upserted_id)
         self.assertTrue(result.acknowledged)
-        self.assertEqual(1, db.test.count({"z": 1}))
-        self.assertEqual(0, db.test.count({"y": 1}))
+        self.assertEqual(1, db.test.count_documents({"z": 1}))
+        self.assertEqual(0, db.test.count_documents({"y": 1}))
         self.assertEqual(db.test.find_one(id1)["z"], 1)
 
         result = db.test.replace_one({"x": 2}, {"y": 2}, True)
@@ -1344,7 +1394,7 @@ class TestCollection(IntegrationTest):
         self.assertTrue(result.modified_count in (None, 0))
         self.assertTrue(isinstance(result.upserted_id, ObjectId))
         self.assertTrue(result.acknowledged)
-        self.assertEqual(1, db.test.count({"y": 2}))
+        self.assertEqual(1, db.test.count_documents({"y": 2}))
 
         db = db.client.get_database(db.name,
                                     write_concern=WriteConcern(w=0))
@@ -1414,7 +1464,7 @@ class TestCollection(IntegrationTest):
         self.assertTrue(result.modified_count in (None, 2))
         self.assertIsNone(result.upserted_id)
         self.assertTrue(result.acknowledged)
-        self.assertEqual(3, db.test.count({"y": 5}))
+        self.assertEqual(3, db.test.count_documents({"y": 5}))
 
         result = db.test.update_many({"x": 5}, {"$set": {"y": 6}})
         self.assertTrue(isinstance(result, UpdateResult))
@@ -1422,7 +1472,7 @@ class TestCollection(IntegrationTest):
         self.assertTrue(result.modified_count in (None, 1))
         self.assertIsNone(result.upserted_id)
         self.assertTrue(result.acknowledged)
-        self.assertEqual(1, db.test.count({"y": 6}))
+        self.assertEqual(1, db.test.count_documents({"y": 6}))
 
         result = db.test.update_many({"x": 2}, {"$set": {"y": 1}}, True)
         self.assertTrue(isinstance(result, UpdateResult))
@@ -1492,7 +1542,7 @@ class TestCollection(IntegrationTest):
         db.create_collection("test", capped=True, size=1000)
 
         db.test.insert_one({"x": 1})
-        self.assertEqual(1, db.test.count())
+        self.assertEqual(1, db.test.count_documents({}))
 
         # Can't remove from capped collection.
         self.assertRaises(OperationFailure, db.test.delete_one, {"x": 1})
@@ -1507,6 +1557,7 @@ class TestCollection(IntegrationTest):
         coll.insert_one({"x": 1})
         self.db.command("getlasterror", w=1, wtimeout=1)
 
+    @ignore_deprecations
     def test_count(self):
         db = self.db
         db.drop_collection("test")
@@ -1521,6 +1572,32 @@ class TestCollection(IntegrationTest):
         self.assertEqual(
             db.test.count({'foo': re.compile(r'ba.*')}), 2)
 
+    def test_count_documents(self):
+        db = self.db
+        db.drop_collection("test")
+        self.addCleanup(db.drop_collection, "test")
+
+        self.assertEqual(db.test.count_documents({}), 0)
+        db.wrong.insert_many([{}, {}])
+        self.assertEqual(db.test.count_documents({}), 0)
+        db.test.insert_many([{}, {}])
+        self.assertEqual(db.test.count_documents({}), 2)
+        db.test.insert_many([{'foo': 'bar'}, {'foo': 'baz'}])
+        self.assertEqual(db.test.count_documents({'foo': 'bar'}), 1)
+        self.assertEqual(
+            db.test.count_documents({'foo': re.compile(r'ba.*')}), 2)
+
+    def test_estimated_document_count(self):
+        db = self.db
+        db.drop_collection("test")
+        self.addCleanup(db.drop_collection, "test")
+
+        self.assertEqual(db.test.estimated_document_count(), 0)
+        db.wrong.insert_many([{}, {}])
+        self.assertEqual(db.test.estimated_document_count(), 0)
+        db.test.insert_many([{}, {}])
+        self.assertEqual(db.test.estimated_document_count(), 2)
+
     def test_aggregate(self):
         db = self.db
         db.drop_collection("test")
@@ -1623,6 +1700,8 @@ class TestCollection(IntegrationTest):
             self.assertTrue(cursor.alive)
 
     @client_context.require_no_mongos
+    @client_context.require_version_max(4, 1, 0)
+    @ignore_deprecations
     def test_parallel_scan(self):
         db = self.db
         db.drop_collection("test")
@@ -1649,7 +1728,9 @@ class TestCollection(IntegrationTest):
 
     @client_context.require_no_mongos
     @client_context.require_version_min(3, 3, 10)
+    @client_context.require_version_max(4, 1, 0)
     @client_context.require_test_commands
+    @ignore_deprecations
     def test_parallel_scan_max_time_ms(self):
             self.client.admin.command("configureFailPoint",
                                       "maxTimeAlwaysTimeOut",
@@ -1690,7 +1771,7 @@ class TestCollection(IntegrationTest):
         for i in range(10):
             db.test.insert_one({"x": i})
 
-        self.assertEqual(10, db.test.count())
+        self.assertEqual(10, db.test.count_documents({}))
 
         total = 0
         for x in db.test.find({}, skip=4, limit=2):
@@ -1710,18 +1791,18 @@ class TestCollection(IntegrationTest):
         self.assertRaises(InvalidName, db.test.rename, "test.")
         self.assertRaises(InvalidName, db.test.rename, "tes..t")
 
-        self.assertEqual(0, db.test.count())
-        self.assertEqual(0, db.foo.count())
+        self.assertEqual(0, db.test.count_documents({}))
+        self.assertEqual(0, db.foo.count_documents({}))
 
         for i in range(10):
             db.test.insert_one({"x": i})
 
-        self.assertEqual(10, db.test.count())
+        self.assertEqual(10, db.test.count_documents({}))
 
         db.test.rename("foo")
 
-        self.assertEqual(0, db.test.count())
-        self.assertEqual(10, db.foo.count())
+        self.assertEqual(0, db.test.count_documents({}))
+        self.assertEqual(10, db.foo.count_documents({}))
 
         x = 0
         for doc in db.foo.find():
@@ -1899,7 +1980,7 @@ class TestCollection(IntegrationTest):
         self.db.test.insert_one({"bar": "foo"})
 
         self.assertEqual(1,
-                         self.db.test.find({"query": {"$ne": None}}).count())
+                         self.db.test.count_documents({"query": {"$ne": None}}))
         self.assertEqual(1,
                          len(list(self.db.test.find({"query": {"$ne": None}})))
                         )
@@ -1919,7 +2000,7 @@ class TestCollection(IntegrationTest):
         self.db.test.drop()
         n_docs = 2100
         self.db.test.insert_many([{} for _ in range(n_docs)])
-        self.assertEqual(n_docs, self.db.test.count())
+        self.assertEqual(n_docs, self.db.test.count_documents({}))
         self.db.test.drop()
 
     def test_map_reduce(self):
@@ -2281,7 +2362,7 @@ class TestCollection(IntegrationTest):
     def test_find_command_generation(self):
         cmd = _gen_find_command('coll', {'$query': {'foo': 1}, '$dumb': 2},
                                 None, 0, 0, 0, None, DEFAULT_READ_CONCERN,
-                                None)
+                                None, None)
         self.assertEqual(
             cmd.to_dict(),
             SON([('find', 'coll'),
