@@ -1,4 +1,4 @@
-# Copyright 2015 MongoDB, Inc.
+# Copyright 2015-present MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,7 +14,6 @@
 
 """Run the command monitoring spec tests."""
 
-import json
 import os
 import re
 import sys
@@ -23,14 +22,13 @@ sys.path[0:0] = [""]
 
 import pymongo
 
-from bson.json_util import object_hook
+from bson import json_util
 from pymongo import monitoring
 from pymongo.errors import OperationFailure
-from pymongo.read_preferences import (make_read_preference,
-                                      read_pref_mode_from_name)
 from pymongo.write_concern import WriteConcern
 from test import unittest, client_context
 from test.utils import single_client, wait_until, EventListener
+from test.utils_selection_tests import parse_read_preference
 
 # Location of JSON test specifications.
 _TEST_PATH = os.path.join(
@@ -82,19 +80,20 @@ def create_test(scenario_def, test):
         coll.insert_many(scenario_def['data'])
         self.listener.results.clear()
         name = camel_to_snake(test['operation']['name'])
-        # Don't send $readPreference to mongos before 2.4.
-        if (client_context.version.at_least(2, 4, 0)
-                and 'read_preference' in test['operation']):
-            mode = read_pref_mode_from_name(
-                test['operation']['read_preference']['mode'])
-            coll = coll.with_options(
-                read_preference=make_read_preference(mode, None))
+        if 'read_preference' in test['operation']:
+            coll = coll.with_options(read_preference=parse_read_preference(
+                test['operation']['read_preference']))
+        if 'collectionOptions' in test['operation']:
+            colloptions = test['operation']['collectionOptions']
+            if 'writeConcern' in colloptions:
+                concern = colloptions['writeConcern']
+                coll = coll.with_options(
+                    write_concern=WriteConcern(**concern))
 
         test_args = test['operation']['arguments']
-        if 'writeConcern' in test_args:
-            concern = test_args.pop('writeConcern')
-            coll = coll.with_options(
-                write_concern=WriteConcern(**concern))
+        if 'options' in test_args:
+            options = test_args.pop('options')
+            test_args.update(options)
         args = {}
         for arg in test_args:
             args[camel_to_snake(arg)] = test_args[arg]
@@ -102,9 +101,9 @@ def create_test(scenario_def, test):
         if name == 'bulk_write':
             bulk_args = []
             for request in args['requests']:
-                opname = next(iter(request))
+                opname = request['name']
                 klass = opname[0:1].upper() + opname[1:]
-                arg = getattr(pymongo, klass)(**request[opname])
+                arg = getattr(pymongo, klass)(**request['arguments'])
                 bulk_args.append(arg)
             try:
                 coll.bulk_write(bulk_args, args.get('ordered', True))
@@ -113,6 +112,9 @@ def create_test(scenario_def, test):
         elif name == 'find':
             if 'sort' in args:
                 args['sort'] = list(args['sort'].items())
+            for arg in 'skip', 'limit':
+                if arg in args:
+                    args[arg] = int(args[arg])
             try:
                 # Iterate the cursor.
                 tuple(coll.find(**args))
@@ -156,6 +158,11 @@ def create_test(scenario_def, test):
                     # and "" for any error message.
                     if 'writeErrors' in reply:
                         for doc in reply['writeErrors']:
+                            # Remove any new fields the server adds. The tests
+                            # only have index, code, and errmsg.
+                            diff = set(doc) - set(['index', 'code', 'errmsg'])
+                            for field in diff:
+                                doc.pop(field)
                             doc['code'] = 42
                             doc['errmsg'] = ""
                     elif 'cursor' in reply:
@@ -187,6 +194,9 @@ def create_test(scenario_def, test):
                         format_actual_results(res)))
 
             for attr, expected in expectation[event_type].items():
+                if 'options' in expected:
+                    options = expected.pop('options')
+                    expected.update(options)
                 actual = getattr(event, attr)
                 if isinstance(expected, dict):
                     for key, val in expected.items():
@@ -202,8 +212,7 @@ def create_tests():
         dirname = os.path.split(dirpath)[-1]
         for filename in filenames:
             with open(os.path.join(dirpath, filename)) as scenario_stream:
-                scenario_def = json.load(
-                    scenario_stream, object_hook=object_hook)
+                scenario_def = json_util.loads(scenario_stream.read())
             assert bool(scenario_def.get('tests')), "tests cannot be empty"
             # Construct test from scenario.
             for test in scenario_def['tests']:
@@ -224,7 +233,7 @@ def create_tests():
                         new_test = client_context.require_no_mongos(None)(
                             new_test)
 
-                test_name = 'test_%s.%s.%s' % (
+                test_name = 'test_%s_%s_%s' % (
                     dirname,
                     os.path.splitext(filename)[0],
                     str(test['description'].replace(" ", "_")))
