@@ -732,6 +732,7 @@ class TestCursor(IntegrationTest):
             break
         self.assertRaises(InvalidOperation, a.sort, "x", ASCENDING)
 
+    @ignore_deprecations
     def test_count(self):
         db = self.db
         db.test.drop()
@@ -756,6 +757,7 @@ class TestCursor(IntegrationTest):
 
         self.assertEqual(0, db.test.acollectionthatdoesntexist.find().count())
 
+    @ignore_deprecations
     def test_count_with_hint(self):
         collection = self.db.test
         collection.drop()
@@ -785,6 +787,7 @@ class TestCursor(IntegrationTest):
             self.assertEqual(2, collection.find().hint("x_1").count())
             self.assertEqual(2, collection.find().hint([("x", 1)]).count())
 
+    @ignore_deprecations
     def test_where(self):
         db = self.db
         db.test.drop()
@@ -954,6 +957,7 @@ class TestCursor(IntegrationTest):
         self.assertTrue(isinstance(cursor2._Cursor__hint, SON))
         self.assertEqual(cursor._Cursor__hint, cursor2._Cursor__hint)
 
+    @ignore_deprecations
     def test_count_with_fields(self):
         self.db.test.drop()
         self.db.test.insert_one({"x": 1})
@@ -1045,6 +1049,7 @@ class TestCursor(IntegrationTest):
         self.assertRaises(IndexError,
                           lambda x: self.db.test.find().skip(50)[x], 50)
 
+    @ignore_deprecations
     def test_count_with_limit_and_skip(self):
         self.assertRaises(TypeError, self.db.test.find().count, "foo")
 
@@ -1122,7 +1127,7 @@ class TestCursor(IntegrationTest):
         # and that the cursor doesn't think it's still alive.
         self.assertFalse(cursor.alive)
 
-        self.assertEqual(3, db.test.count())
+        self.assertEqual(3, db.test.count_documents({}))
 
         # __getitem__(index)
         for cursor in (db.test.find(cursor_type=CursorType.TAILABLE),
@@ -1188,6 +1193,7 @@ class TestCursor(IntegrationTest):
 
         self.assertEqual(["b", "c"], distinct)
 
+    @client_context.require_version_max(4, 1, 0, -1)
     def test_max_scan(self):
         self.db.drop_collection("test")
         self.db.test.insert_many([{} for _ in range(100)])
@@ -1212,12 +1218,13 @@ class TestCursor(IntegrationTest):
         self.assertTrue(c1.alive)
 
     @client_context.require_no_mongos
+    @ignore_deprecations
     def test_comment(self):
         if client_context.auth_enabled:
             raise SkipTest("SERVER-4754 - This test uses profiling.")
 
         # MongoDB 3.1.5 changed the ns for commands.
-        regex = {'$regex': 'pymongo_test.(\$cmd|test)'}
+        regex = {'$regex': r'pymongo_test.(\$cmd|test)'}
 
         if client_context.version.at_least(3, 5, 8, -1):
             query_key = "command.comment"
@@ -1298,9 +1305,6 @@ class TestCursor(IntegrationTest):
             cur = c.find(hint=[("a", 1)], modifiers={"$hint": {"b": "1"}})
             self.assertEqual(cur._Cursor__query_spec()["$hint"], {"a": 1})
 
-            cur = c.find(snapshot=True, modifiers={"$snapshot": False})
-            self.assertEqual(cur._Cursor__query_spec()["$snapshot"], True)
-
             # The arg is named show_record_id after the "find" command arg, the
             # modifier is named $showDiskLoc for the OP_QUERY modifier. It's
             # stored as $showDiskLoc then upgraded to showRecordId if we send a
@@ -1308,6 +1312,10 @@ class TestCursor(IntegrationTest):
             cur = c.find(show_record_id=True, modifiers={"$showDiskLoc": False})
             self.assertEqual(cur._Cursor__query_spec()["$showDiskLoc"], True)
 
+            if not client_context.version.at_least(3, 7, 3):
+                cur = c.find(snapshot=True, modifiers={"$snapshot": False})
+                self.assertEqual(cur._Cursor__query_spec()["$snapshot"], True)
+
     def test_alive(self):
         self.db.test.delete_many({})
         self.db.test.insert_many([{} for _ in range(3)])
