@@ -45,6 +45,7 @@ from pymongo.errors import (CollectionInvalid,
 from pymongo.mongo_client import MongoClient
 from pymongo.read_concern import ReadConcern
 from pymongo.read_preferences import ReadPreference
+from pymongo.saslprep import HAVE_STRINGPREP
 from pymongo.write_concern import WriteConcern
 from test import (client_context,
                   SkipTest,
@@ -146,13 +147,13 @@ class TestDatabase(IntegrationTest):
         self.assertRaises(InvalidName, db.create_collection, "coll..ection")
 
         test = db.create_collection("test")
-        self.assertTrue(u"test" in db.collection_names())
+        self.assertTrue(u"test" in db.list_collection_names())
         test.insert_one({"hello": u"world"})
         self.assertEqual(db.test.find_one()["hello"], "world")
 
         db.drop_collection("test.foo")
         db.create_collection("test.foo")
-        self.assertTrue(u"test.foo" in db.collection_names())
+        self.assertTrue(u"test.foo" in db.list_collection_names())
         self.assertRaises(CollectionInvalid, db.create_collection, "test.foo")
 
     def _test_collection_names(self, meth, test_no_system):
@@ -273,7 +274,7 @@ class TestDatabase(IntegrationTest):
         for i in range(200):
             db.create_collection(str(i))
 
-        db.collection_names()  # Must not hang.
+        db.list_collection_names()  # Must not hang.
         client.drop_database('test_collection_names_single_socket')
 
     def test_drop_collection(self):
@@ -283,24 +284,24 @@ class TestDatabase(IntegrationTest):
         self.assertRaises(TypeError, db.drop_collection, None)
 
         db.test.insert_one({"dummy": u"object"})
-        self.assertTrue("test" in db.collection_names())
+        self.assertTrue("test" in db.list_collection_names())
         db.drop_collection("test")
-        self.assertFalse("test" in db.collection_names())
+        self.assertFalse("test" in db.list_collection_names())
 
         db.test.insert_one({"dummy": u"object"})
-        self.assertTrue("test" in db.collection_names())
+        self.assertTrue("test" in db.list_collection_names())
         db.drop_collection(u"test")
-        self.assertFalse("test" in db.collection_names())
+        self.assertFalse("test" in db.list_collection_names())
 
         db.test.insert_one({"dummy": u"object"})
-        self.assertTrue("test" in db.collection_names())
+        self.assertTrue("test" in db.list_collection_names())
         db.drop_collection(db.test)
-        self.assertFalse("test" in db.collection_names())
+        self.assertFalse("test" in db.list_collection_names())
 
         db.test.insert_one({"dummy": u"object"})
-        self.assertTrue("test" in db.collection_names())
+        self.assertTrue("test" in db.list_collection_names())
         db.test.drop()
-        self.assertFalse("test" in db.collection_names())
+        self.assertFalse("test" in db.list_collection_names())
         db.test.drop()
 
         db.drop_collection(db.test.doesnotexist)
@@ -416,7 +417,7 @@ class TestDatabase(IntegrationTest):
             # getLastError includes "connectionId" in recent
             # server versions, getPrevError does not.
             error.pop("connectionId", None)
-            self.assertEqual(error, prev_error)
+            self.assertEqualReply(error, prev_error)
 
             db.test.find_one()
             self.assertEqual(None, db.error())
@@ -503,12 +504,8 @@ class TestDatabase(IntegrationTest):
             self.assertRaises(ConfigurationError, auth_db.add_user,
                               "user", "password", digestPassword=True)
 
-        extra = {}
-        if client_context.version.at_least(3, 7, 2):
-            extra['mechanisms'] = ['SCRAM-SHA-1']
-
         # Add / authenticate / remove
-        auth_db.add_user("mike", "password", roles=["read"], **extra)
+        auth_db.add_user("mike", "password", roles=["read"])
         self.addCleanup(remove_all_users, auth_db)
         self.assertRaises(TypeError, check_auth, 5, "password")
         self.assertRaises(TypeError, check_auth, "mike", 5)
@@ -517,35 +514,34 @@ class TestDatabase(IntegrationTest):
         self.assertRaises(OperationFailure, check_auth, "faker", "password")
         check_auth("mike", "password")
 
-        # Unicode name and password.
-        check_auth(u"mike", u"password")
+        if not client_context.version.at_least(3, 7, 2) or HAVE_STRINGPREP:
+            # Unicode name and password.
+            check_auth(u"mike", u"password")
 
-        auth_db.remove_user("mike")
-        self.assertRaises(OperationFailure,
-                          check_auth, "mike", "password")
+            auth_db.remove_user("mike")
+            self.assertRaises(
+                OperationFailure, check_auth, "mike", "password")
 
-        # Add / authenticate / change password
-        self.assertRaises(OperationFailure, check_auth, "Gustave", u"Dor\xe9")
-        auth_db.add_user("Gustave", u"Dor\xe9", roles=["read"], **extra)
-        check_auth("Gustave", u"Dor\xe9")
+            # Add / authenticate / change password
+            self.assertRaises(
+                OperationFailure, check_auth, "Gustave", u"Dor\xe9")
+            auth_db.add_user("Gustave", u"Dor\xe9", roles=["read"])
+            check_auth("Gustave", u"Dor\xe9")
 
-        # Change password.
-        auth_db.add_user("Gustave", "password", roles=["read"], **extra)
-        self.assertRaises(OperationFailure, check_auth, "Gustave", u"Dor\xe9")
-        check_auth("Gustave", u"password")
+            # Change password.
+            auth_db.add_user("Gustave", "password", roles=["read"])
+            self.assertRaises(
+                OperationFailure, check_auth, "Gustave", u"Dor\xe9")
+            check_auth("Gustave", u"password")
 
     @client_context.require_auth
     @ignore_deprecations
     def test_make_user_readonly(self):
-        extra = {}
-        if client_context.version.at_least(3, 7, 2):
-            extra['mechanisms'] = ['SCRAM-SHA-1']
-
         # "self.client" is logged in as root.
         auth_db = self.client.pymongo_test
 
         # Make a read-write user.
-        auth_db.add_user('jesse', 'pw', **extra)
+        auth_db.add_user('jesse', 'pw')
         self.addCleanup(remove_all_users, auth_db)
 
         # Check that we're read-write by default.
@@ -556,7 +552,7 @@ class TestDatabase(IntegrationTest):
         c.pymongo_test.collection.insert_one({})
 
         # Make the user read-only.
-        auth_db.add_user('jesse', 'pw', read_only=True, **extra)
+        auth_db.add_user('jesse', 'pw', read_only=True)
 
         c = rs_or_single_client_noauth(username='jesse',
                                        password='pw',
@@ -569,13 +565,9 @@ class TestDatabase(IntegrationTest):
     @client_context.require_auth
     @ignore_deprecations
     def test_default_roles(self):
-        extra = {}
-        if client_context.version.at_least(3, 7, 2):
-            extra['mechanisms'] = ['SCRAM-SHA-1']
-
         # "self.client" is logged in as root.
         auth_admin = self.client.admin
-        auth_admin.add_user('test_default_roles', 'pass', **extra)
+        auth_admin.add_user('test_default_roles', 'pass')
         self.addCleanup(client_context.drop_user, 'admin', 'test_default_roles')
         info = auth_admin.command(
             'usersInfo', 'test_default_roles')['users'][0]
@@ -583,33 +575,29 @@ class TestDatabase(IntegrationTest):
         self.assertEqual("root", info['roles'][0]['role'])
 
         # Read only "admin" user
-        auth_admin.add_user('ro-admin', 'pass', read_only=True, **extra)
+        auth_admin.add_user('ro-admin', 'pass', read_only=True)
         self.addCleanup(client_context.drop_user, 'admin', 'ro-admin')
         info = auth_admin.command('usersInfo', 'ro-admin')['users'][0]
         self.assertEqual("readAnyDatabase", info['roles'][0]['role'])
 
         # "Non-admin" user
         auth_db = self.client.pymongo_test
-        auth_db.add_user('user', 'pass', **extra)
+        auth_db.add_user('user', 'pass')
         self.addCleanup(remove_all_users, auth_db)
         info = auth_db.command('usersInfo', 'user')['users'][0]
         self.assertEqual("dbOwner", info['roles'][0]['role'])
 
         # Read only "Non-admin" user
-        auth_db.add_user('ro-user', 'pass', read_only=True, **extra)
+        auth_db.add_user('ro-user', 'pass', read_only=True)
         info = auth_db.command('usersInfo', 'ro-user')['users'][0]
         self.assertEqual("read", info['roles'][0]['role'])
 
     @client_context.require_auth
     @ignore_deprecations
     def test_new_user_cmds(self):
-        extra = {}
-        if client_context.version.at_least(3, 7, 2):
-            extra['mechanisms'] = ['SCRAM-SHA-1']
-
         # "self.client" is logged in as root.
         auth_db = self.client.pymongo_test
-        auth_db.add_user("amalia", "password", roles=["userAdmin"], **extra)
+        auth_db.add_user("amalia", "password", roles=["userAdmin"])
         self.addCleanup(client_context.drop_user, "pymongo_test", "amalia")
 
         db = rs_or_single_client_noauth(username="amalia",
@@ -618,7 +606,7 @@ class TestDatabase(IntegrationTest):
 
         # This tests the ability to update user attributes.
         db.add_user("amalia", "new_password",
-                    customData={"secret": "koalas"}, **extra)
+                    customData={"secret": "koalas"})
 
         user_info = db.command("usersInfo", "amalia")
         self.assertTrue(user_info["users"])
@@ -629,10 +617,6 @@ class TestDatabase(IntegrationTest):
     @client_context.require_auth
     @ignore_deprecations
     def test_authenticate_multiple(self):
-        extra = {}
-        if client_context.version.at_least(3, 7, 2):
-            extra['mechanisms'] = ['SCRAM-SHA-1']
-
         # "self.client" is logged in as root.
         self.client.drop_database("pymongo_test")
         self.client.drop_database("pymongo_test1")
@@ -650,18 +634,17 @@ class TestDatabase(IntegrationTest):
         admin_db_auth.add_user(
             'ro-admin',
             'pass',
-            roles=["userAdmin", "readAnyDatabase"],
-            **extra)
+            roles=["userAdmin", "readAnyDatabase"])
 
         self.addCleanup(client_context.drop_user, 'admin', 'ro-admin')
         users_db_auth.add_user(
-            'user', 'pass', roles=["userAdmin", "readWrite"], **extra)
+            'user', 'pass', roles=["userAdmin", "readWrite"])
         self.addCleanup(remove_all_users, users_db_auth)
 
         # Regular user should be able to query its own db, but
         # no other.
         users_db.authenticate('user', 'pass')
-        self.assertEqual(0, users_db.test.count())
+        self.assertEqual(0, users_db.test.count_documents({}))
         self.assertRaises(OperationFailure, other_db.test.find_one)
 
         # Admin read-only user should be able to query any db,
@@ -678,7 +661,7 @@ class TestDatabase(IntegrationTest):
         self.assertTrue(users_db.test.delete_many({}))
 
         # And read from other dbs...
-        self.assertEqual(0, other_db.test.count())
+        self.assertEqual(0, other_db.test.count_documents({}))
 
         # But still not write to other dbs.
         self.assertRaises(OperationFailure,
@@ -738,6 +721,7 @@ class TestDatabase(IntegrationTest):
                                         projection={"_id": False}))
 
     @client_context.require_no_auth
+    @client_context.require_version_max(4, 1, 0)
     def test_eval(self):
         db = self.client.pymongo_test
         db.test.drop()
@@ -841,24 +825,25 @@ class TestDatabase(IntegrationTest):
         self.assertFalse(db.test.find_one())
 
     @client_context.require_no_auth
+    @client_context.require_version_max(4, 1, 0)
     def test_system_js(self):
         db = self.client.pymongo_test
         db.system.js.delete_many({})
 
-        self.assertEqual(0, db.system.js.count())
+        self.assertEqual(0, db.system.js.count_documents({}))
         db.system_js.add = "function(a, b) { return a + b; }"
         self.assertEqual('add', db.system.js.find_one()['_id'])
-        self.assertEqual(1, db.system.js.count())
+        self.assertEqual(1, db.system.js.count_documents({}))
         self.assertEqual(6, db.system_js.add(1, 5))
         del db.system_js.add
-        self.assertEqual(0, db.system.js.count())
+        self.assertEqual(0, db.system.js.count_documents({}))
 
         db.system_js['add'] = "function(a, b) { return a + b; }"
         self.assertEqual('add', db.system.js.find_one()['_id'])
-        self.assertEqual(1, db.system.js.count())
+        self.assertEqual(1, db.system.js.count_documents({}))
         self.assertEqual(6, db.system_js['add'](1, 5))
         del db.system_js['add']
-        self.assertEqual(0, db.system.js.count())
+        self.assertEqual(0, db.system.js.count_documents({}))
         self.assertRaises(OperationFailure, db.system_js.add, 1, 5)
 
         # TODO right now CodeWScope doesn't work w/ system js
