@@ -1,4 +1,4 @@
-# Copyright 2014-2015 MongoDB, Inc.
+# Copyright 2014-present MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -20,8 +20,9 @@ import threading
 
 sys.path[0:0] = [""]
 
-from bson import json_util
+from bson import json_util, Timestamp
 from pymongo import common
+from pymongo.errors import ConfigurationError
 from pymongo.topology import Topology
 from pymongo.topology_description import TOPOLOGY_TYPE
 from pymongo.ismaster import IsMaster
@@ -126,6 +127,13 @@ def check_outcome(self, topology, outcome):
         len(topology.description.server_descriptions()),
         len(expected_servers))
 
+    if outcome.get('compatible') is False:
+        with self.assertRaises(ConfigurationError):
+            topology.description.check_compatible()
+    else:
+        # No error.
+        topology.description.check_compatible()
+
     # Since lengths are equal, every actual server must have a corresponding
     # expected server.
     for expected_server_address, expected_server in expected_servers.items():
@@ -148,7 +156,7 @@ def check_outcome(self, topology, outcome):
             server_type_name(actual_server_description.server_type))
 
         self.assertEqual(
-            expected_server['setName'],
+            expected_server.get('setName'),
             actual_server_description.replica_set_name)
 
         self.assertEqual(
@@ -160,6 +168,8 @@ def check_outcome(self, topology, outcome):
             actual_server_description.election_id)
 
     self.assertEqual(outcome['setName'], topology.description.replica_set_name)
+    self.assertEqual(outcome['logicalSessionTimeoutMinutes'],
+                     topology.description.logical_session_timeout_minutes)
     expected_topology_type = getattr(TOPOLOGY_TYPE, outcome['topologyType'])
     self.assertEqual(topology_type_name(expected_topology_type),
                      topology_type_name(topology.description.topology_type))
@@ -168,7 +178,7 @@ def check_outcome(self, topology, outcome):
 def create_test(scenario_def):
     def run_scenario(self):
         c = create_mock_topology(scenario_def['uri'])
-        
+
         for phase in scenario_def['phases']:
             for response in phase['responses']:
                 got_ismaster(c,
@@ -199,5 +209,33 @@ def create_tests():
 
 create_tests()
 
+
+class TestClusterTimeComparison(unittest.TestCase):
+    def test_cluster_time_comparison(self):
+        t = create_mock_topology('mongodb://host')
+
+        def send_cluster_time(time, inc, should_update):
+            old = t.max_cluster_time()
+            new = {'clusterTime': Timestamp(time, inc)}
+            got_ismaster(t,
+                         ('host', 27017),
+                         {'ok': 1,
+                          'minWireVersion': 0,
+                          'maxWireVersion': 6,
+                          '$clusterTime': new})
+
+            actual = t.max_cluster_time()
+            if should_update:
+                self.assertEqual(actual, new)
+            else:
+                self.assertEqual(actual, old)
+
+        send_cluster_time(0, 1, True)
+        send_cluster_time(2, 2, True)
+        send_cluster_time(2, 1, False)
+        send_cluster_time(1, 3, False)
+        send_cluster_time(2, 3, True)
+
+
 if __name__ == "__main__":
     unittest.main()
