@@ -19,7 +19,12 @@ import sys
 
 sys.path[0:0] = [""]
 
-from test import client_context, unittest
+import pymongo
+from pymongo.errors import ConnectionFailure, OperationFailure
+from pymongo.read_concern import ReadConcern
+from pymongo.write_concern import WriteConcern
+from test import client_context, unittest, IntegrationTest
+from test.utils import rs_or_single_client
 
 
 class TestSampleShellCommands(unittest.TestCase):
@@ -27,12 +32,17 @@ class TestSampleShellCommands(unittest.TestCase):
     @classmethod
     @client_context.require_connection
     def setUpClass(cls):
+        cls.client = rs_or_single_client(w="majority")
         # Run once before any tests run.
-        client_context.client.pymongo_test.inventory.drop()
+        cls.client.pymongo_test.inventory.drop()
+
+    @classmethod
+    def tearDownClass(cls):
+        client_context.client.drop_database("pymongo_test")
 
     def tearDown(self):
         # Run after every test.
-        client_context.client.pymongo_test.inventory.drop()
+        self.client.pymongo_test.inventory.drop()
 
     def test_first_three_examples(self):
         db = client_context.client.pymongo_test
@@ -45,7 +55,7 @@ class TestSampleShellCommands(unittest.TestCase):
              "size": {"h": 28, "w": 35.5, "uom": "cm"}})
         # End Example 1
 
-        self.assertEqual(db.inventory.count(), 1)
+        self.assertEqual(db.inventory.count_documents({}), 1)
 
         # Start Example 2
         cursor = db.inventory.find({"item": "canvas"})
@@ -69,7 +79,7 @@ class TestSampleShellCommands(unittest.TestCase):
              "size": {"h": 19, "w": 22.85, "uom": "cm"}}])
         # End Example 3
 
-        self.assertEqual(db.inventory.count(), 4)
+        self.assertEqual(db.inventory.count_documents({}), 4)
 
     def test_query_top_level_fields(self):
         db = client_context.client.pymongo_test
@@ -97,7 +107,7 @@ class TestSampleShellCommands(unittest.TestCase):
              "status": "A"}])
         # End Example 6
 
-        self.assertEqual(db.inventory.count(), 5)
+        self.assertEqual(db.inventory.count_documents({}), 5)
 
         # Start Example 7
         cursor = db.inventory.find({})
@@ -628,25 +638,25 @@ class TestSampleShellCommands(unittest.TestCase):
              "status": "A"}])
         # End Example 55
 
-        self.assertEqual(db.inventory.count(), 5)
+        self.assertEqual(db.inventory.count_documents({}), 5)
 
         # Start Example 57
         db.inventory.delete_many({"status": "A"})
         # End Example 57
 
-        self.assertEqual(db.inventory.count(), 3)
+        self.assertEqual(db.inventory.count_documents({}), 3)
 
         # Start Example 58
         db.inventory.delete_one({"status": "D"})
         # End Example 58
 
-        self.assertEqual(db.inventory.count(), 2)
+        self.assertEqual(db.inventory.count_documents({}), 2)
 
         # Start Example 56
         db.inventory.delete_many({})
         # End Example 56
 
-        self.assertEqual(db.inventory.count(), 0)
+        self.assertEqual(db.inventory.count_documents({}), 0)
 
     @client_context.require_version_min(3, 5, 11)
     @client_context.require_replica_set
@@ -682,8 +692,8 @@ class TestSampleShellCommands(unittest.TestCase):
 
             # Start Changestream Example 4
             pipeline = [
-                {"$match":
-                    {"$or": [
+                {"$match": {
+                    "$or": [
                         {"fullDocument.username": "alice"},
                         {"operationType": {"$in": ["delete"]}}]
                     }
@@ -696,6 +706,118 @@ class TestSampleShellCommands(unittest.TestCase):
             done = True
             t.join()
 
+    def test_aggregate_examples(self):
+        db = client_context.client.pymongo_test
+
+        # Start Aggregation Example 1
+        db.sales.aggregate([
+            {"$match": {"items.fruit": "banana"}},
+            {"$sort": {"date": 1}}
+        ])
+        # End Aggregation Example 1
+
+        # Start Aggregation Example 2
+        db.sales.aggregate([
+            {"$unwind": "$items"},
+            {"$match": {"items.fruit": "banana"}},
+            {"$group": {
+                "_id": {"day": {"$dayOfWeek": "$date"}},
+                "count": {"$sum": "$items.quantity"}}
+            },
+            {"$project": {
+                "dayOfWeek": "$_id.day",
+                "numberSold": "$count",
+                "_id": 0}
+            },
+            {"$sort": {"numberSold": 1}}
+        ])
+        # End Aggregation Example 2
+
+        # Start Aggregation Example 3
+        db.sales.aggregate([
+            {"$unwind": "$items"},
+            {"$group": {
+                "_id": {"day": {"$dayOfWeek": "$date"}},
+                "items_sold": {"$sum": "$items.quantity"},
+                "revenue": {
+                    "$sum": {
+                        "$multiply": [
+                            "$items.quantity", "$items.price"]
+                        }
+                    }
+                }
+            },
+            {"$project": {
+                "day": "$_id.day",
+                "revenue": 1,
+                "items_sold": 1,
+                "discount": {
+                    "$cond": {
+                        "if": {"$lte": ["$revenue", 250]},
+                        "then": 25,
+                        "else": 0
+                        }
+                    }
+                }
+            }
+        ])
+        # End Aggregation Example 3
+
+        # $lookup was new in 3.2. The let and pipeline options
+        # were added in 3.6.
+        if client_context.version.at_least(3, 6, 0):
+            # Start Aggregation Example 4
+            db.air_alliances.aggregate([
+                {"$lookup": {
+                    "from": "air_airlines",
+                    "let": {"constituents": "$airlines"},
+                    "pipeline": [
+                        {"$match": {"$expr": {"$in": ["$name", "$$constituents"]}}}
+                    ],
+                    "as": "airlines"
+                    }
+                },
+                {"$project": {
+                    "_id": 0,
+                    "name": 1,
+                    "airlines": {
+                        "$filter": {
+                            "input": "$airlines",
+                            "as": "airline",
+                            "cond": {"$eq": ["$$airline.country", "Canada"]}
+                            }
+                        }
+                    }
+                }
+            ])
+            # End Aggregation Example 4
+
+    def test_commands(self):
+        db = client_context.client.pymongo_test
+        db.restaurants.insert_one({})
+
+        # Start runCommand Example 1
+        db.command("buildInfo")
+        # End runCommand Example 1
+
+        # Start runCommand Example 2
+        db.command("collStats", "restaurants")
+        # End runCommand Example 2
+
+    def test_index_management(self):
+        db = client_context.client.pymongo_test
+
+        # Start Index Example 1
+        db.records.create_index("score")
+        # End Index Example 1
+
+        # Start Index Example 1
+        db.restaurants.create_index(
+            [("cuisine", pymongo.ASCENDING), ("name", pymongo.ASCENDING)],
+            partialFilterExpression={"rating": {"$gt": 5}}
+        )
+        # End Index Example 1
+
     @client_context.require_version_min(3, 6, 0)
     @client_context.require_replica_set
     def test_misc(self):
@@ -721,5 +843,276 @@ class TestSampleShellCommands(unittest.TestCase):
             array_filters=[{"i.b": 0}])
 
 
+class TestTransactionExamples(IntegrationTest):
+
+    @classmethod
+    @client_context.require_connection
+    def setUpClass(cls):
+        super(TestTransactionExamples, cls).setUpClass()
+        cls.client = rs_or_single_client(w="majority")
+
+    @client_context.require_transactions
+    def test_transactions(self):
+        # Transaction examples
+        client = self.client
+        self.addCleanup(client.drop_database, "hr")
+        self.addCleanup(client.drop_database, "reporting")
+
+        employees = client.hr.employees
+        events = client.reporting.events
+        employees.insert_one({"employee": 3, "status": "Active"})
+        events.insert_one(
+            {"employee": 3, "status": {"new": "Active", "old": None}})
+
+        # Start Transactions Intro Example 1
+
+        def update_employee_info(session):
+            employees_coll = session.client.hr.employees
+            events_coll = session.client.reporting.events
+
+            with session.start_transaction(
+                    read_concern=ReadConcern("snapshot"),
+                    write_concern=WriteConcern(w="majority")):
+                employees_coll.update_one(
+                    {"employee": 3}, {"$set": {"status": "Inactive"}},
+                    session=session)
+                events_coll.insert_one(
+                    {"employee": 3, "status": {
+                        "new": "Inactive", "old": "Active"}},
+                    session=session)
+
+                while True:
+                    try:
+                        # Commit uses write concern set at transaction start.
+                        session.commit_transaction()
+                        print("Transaction committed.")
+                        break
+                    except (ConnectionFailure, OperationFailure) as exc:
+                        # Can retry commit
+                        if exc.has_error_label(
+                                "UnknownTransactionCommitResult"):
+                            print("UnknownTransactionCommitResult, retrying "
+                                  "commit operation ...")
+                            continue
+                        else:
+                            print("Error during commit ...")
+                            raise
+        # End Transactions Intro Example 1
+
+        with client.start_session() as session:
+            update_employee_info(session)
+
+        employee = employees.find_one({"employee": 3})
+        self.assertIsNotNone(employee)
+        self.assertEqual(employee['status'], 'Inactive')
+
+        # Start Transactions Retry Example 1
+        def run_transaction_with_retry(txn_func, session):
+            while True:
+                try:
+                    txn_func(session)  # performs transaction
+                    break
+                except (ConnectionFailure, OperationFailure) as exc:
+                    print("Transaction aborted. Caught exception during "
+                          "transaction.")
+
+                    # If transient error, retry the whole transaction
+                    if exc.has_error_label("TransientTransactionError"):
+                        print("TransientTransactionError, retrying"
+                              "transaction ...")
+                        continue
+                    else:
+                        raise
+        # End Transactions Retry Example 1
+
+        with client.start_session() as session:
+            run_transaction_with_retry(update_employee_info, session)
+
+        employee = employees.find_one({"employee": 3})
+        self.assertIsNotNone(employee)
+        self.assertEqual(employee['status'], 'Inactive')
+
+        # Start Transactions Retry Example 2
+        def commit_with_retry(session):
+            while True:
+                try:
+                    # Commit uses write concern set at transaction start.
+                    session.commit_transaction()
+                    print("Transaction committed.")
+                    break
+                except (ConnectionFailure, OperationFailure) as exc:
+                    # Can retry commit
+                    if exc.has_error_label("UnknownTransactionCommitResult"):
+                        print("UnknownTransactionCommitResult, retrying "
+                              "commit operation ...")
+                        continue
+                    else:
+                        print("Error during commit ...")
+                        raise
+        # End Transactions Retry Example 2
+
+        # Test commit_with_retry from the previous examples
+        def _insert_employee_retry_commit(session):
+            with session.start_transaction():
+                employees.insert_one(
+                    {"employee": 4, "status": "Active"},
+                    session=session)
+                events.insert_one(
+                    {"employee": 4, "status": {"new": "Active", "old": None}},
+                    session=session)
+
+                commit_with_retry(session)
+
+        with client.start_session() as session:
+            run_transaction_with_retry(_insert_employee_retry_commit, session)
+
+        employee = employees.find_one({"employee": 4})
+        self.assertIsNotNone(employee)
+        self.assertEqual(employee['status'], 'Active')
+
+        # Start Transactions Retry Example 3
+
+        def run_transaction_with_retry(txn_func, session):
+            while True:
+                try:
+                    txn_func(session)  # performs transaction
+                    break
+                except (ConnectionFailure, OperationFailure) as exc:
+                    # If transient error, retry the whole transaction
+                    if exc.has_error_label("TransientTransactionError"):
+                        print("TransientTransactionError, retrying "
+                              "transaction ...")
+                        continue
+                    else:
+                        raise
+
+        def commit_with_retry(session):
+            while True:
+                try:
+                    # Commit uses write concern set at transaction start.
+                    session.commit_transaction()
+                    print("Transaction committed.")
+                    break
+                except (ConnectionFailure, OperationFailure) as exc:
+                    # Can retry commit
+                    if exc.has_error_label("UnknownTransactionCommitResult"):
+                        print("UnknownTransactionCommitResult, retrying "
+                              "commit operation ...")
+                        continue
+                    else:
+                        print("Error during commit ...")
+                        raise
+
+        # Updates two collections in a transactions
+
+        def update_employee_info(session):
+            employees_coll = session.client.hr.employees
+            events_coll = session.client.reporting.events
+
+            with session.start_transaction(
+                    read_concern=ReadConcern("snapshot"),
+                    write_concern=WriteConcern(w="majority")):
+                employees_coll.update_one(
+                    {"employee": 3}, {"$set": {"status": "Inactive"}},
+                    session=session)
+                events_coll.insert_one(
+                    {"employee": 3, "status": {
+                        "new": "Inactive", "old": "Active"}},
+                    session=session)
+
+                commit_with_retry(session)
+
+        # Start a session.
+        with client.start_session() as session:
+            try:
+                run_transaction_with_retry(update_employee_info, session)
+            except Exception as exc:
+                # Do something with error.
+                raise
+
+        # End Transactions Retry Example 3
+
+        employee = employees.find_one({"employee": 3})
+        self.assertIsNotNone(employee)
+        self.assertEqual(employee['status'], 'Inactive')
+
+    @client_context.require_transactions
+    def test_transactions_beta(self):
+        # Transaction beta examples
+        client = self.client
+        self.addCleanup(client.drop_database, "test")
+
+        db = client.test
+        shipment = db.create_collection("shipment")
+        inventory = db.create_collection("inventory")
+        inventory.insert_one({"sku": "abc123", "qty": 500})
+
+        # Start Beta Transaction Example 1
+        db = client.test
+        with client.start_session() as s:
+            with s.start_transaction():
+                db.inventory.update_one({'sku': 'abc123'},
+                                        {'$inc': {'qty': -100}},
+                                        session=s)
+                db.shipment.insert_one({'sku': 'abc123', 'qty': 100},
+                                       session=s)
+        # End Beta Transaction Example 1
+
+        # Beta Transaction Example 1 with explicit start, commit, and abort.
+        with client.start_session() as s:
+            s.start_transaction()
+            try:
+                db.inventory.update_one({'sku': 'abc123'},
+                                        {'$inc': {'qty': -100}},
+                                        session=s)
+                db.shipment.insert_one({'sku': 'abc123', 'qty': 100},
+                                       session=s)
+            except Exception:
+                s.abort_transaction()
+                raise
+            s.commit_transaction()
+
+        # Start Beta Transaction Example 2
+        db = client.test
+        shipment = db.get_collection('shipment',
+                                     write_concern=WriteConcern(w='majority'))
+
+        # In the following block, the following write concerns are used:
+        # the update_one and insert_one operations uses w = 1,
+        # the transaction commit/abort uses w = 'majority'.
+        with client.start_session() as s:
+            with s.start_transaction(write_concern=WriteConcern(w='majority')):
+                inventory.update_one({'sku': 'abc123'},
+                                     {'$inc': {'qty': -100}},
+                                     session=s)
+                shipment.insert_one({'sku': 'abc123', 'qty': 100}, session=s)
+        # End Beta Transaction Example 2
+
+        # Start Beta Transaction Example 3
+        def run_transaction(session, txn_callback):
+            with session.start_transaction():
+                txn_callback(session)
+
+        def run_transaction_with_retry(session, txn_callback):
+            try:
+                run_transaction(session, txn_callback)
+            except (OperationFailure, ConnectionFailure) as exc:
+                if exc.has_error_label("TransientTransactionError"):
+                    # Retry the entire transaction on temporary transaction
+                    # failures.
+                    run_transaction(session, txn_callback)
+                else:
+                    raise
+
+        def shipment_transaction(session):
+            inventory.update_one({'sku': 'abc123'}, {'$inc': {'qty': -100}},
+                                 session=session)
+            shipment.insert_one({'sku': 'abc123', 'qty': 100}, session=session)
+
+        with client.start_session() as session:
+            run_transaction_with_retry(session, shipment_transaction)
+        # End Beta Transaction Example 3
+
+
 if __name__ == "__main__":
     unittest.main()
