@@ -271,7 +271,6 @@ class TestCommandMonitoring(PyMongoTestCase):
     def test_find_options(self):
         query = dict(filter={},
                      hint=[('x', 1)],
-                     max_scan=10,
                      max_time_ms=10000,
                      max={'x': 10},
                      min={'x': -10},
@@ -289,7 +288,6 @@ class TestCommandMonitoring(PyMongoTestCase):
                    filter={},
                    hint=SON([('x', 1)]),
                    comment='this is a test',
-                   maxScan=10,
                    maxTimeMS=10000,
                    max={'x': 10},
                    min={'x': -10},
@@ -302,8 +300,13 @@ class TestCommandMonitoring(PyMongoTestCase):
                    noCursorTimeout=True,
                    allowPartialResults=True)
 
+        if client_context.version < (4, 1, 0, -1):
+            query['max_scan'] = 10
+            cmd['maxScan'] = 10
+
         self._test_find_options(query, cmd)
 
+    @client_context.require_version_max(3, 7, 2)
     def test_find_snapshot(self):
         # Test "snapshot" parameter separately, can't combine with "sort".
         query = dict(filter={},
@@ -779,7 +782,7 @@ class TestCommandMonitoring(PyMongoTestCase):
         self.assertEqual(1, reply.get('ok'))
         self.assertEqual(1, reply.get('n'))
 
-        self.assertEqual(0, coll.count())
+        self.assertEqual(0, coll.count_documents({}))
 
         # write errors
         coll.insert_one({'_id': 1})
@@ -1039,7 +1042,7 @@ class TestCommandMonitoring(PyMongoTestCase):
             self.assertEqual(1, reply.get('ok'))
             self.assertEqual(1, reply.get('n'))
 
-            self.assertEqual(0, coll.count())
+            self.assertEqual(0, coll.count_documents({}))
 
     def test_insert_many(self):
         # This always uses the bulk API.
