@@ -37,13 +37,18 @@ from pymongo.server_selectors import readable_server_selector, Selection
 from pymongo.server_type import SERVER_TYPE
 from pymongo.write_concern import WriteConcern
 
-from test.test_replica_set_client import TestReplicaSetClientBase
 from test import (SkipTest,
                   client_context,
                   unittest,
                   db_user,
                   db_pwd)
-from test.utils import connected, single_client, one, wait_until, rs_client
+from test.test_replica_set_client import TestReplicaSetClientBase
+from test.utils import (connected,
+                        ignore_deprecations,
+                        one,
+                        rs_client,
+                        single_client,
+                        wait_until)
 from test.version import Version
 
 
@@ -155,11 +160,12 @@ class TestSingleSlaveOk(TestReadPreferencesBase):
 
         # Test some database helpers.
         self.assertIsNotNone(db.collection_names())
+        self.assertIsNotNone(db.list_collection_names())
         self.assertIsNotNone(db.validate_collection("test"))
-        self.assertIsNotNone(db.command("count", "test"))
+        self.assertIsNotNone(db.command("ping"))
 
         # Test some collection helpers.
-        self.assertEqual(10, coll.count())
+        self.assertEqual(10, coll.count_documents({}))
         self.assertEqual(10, len(coll.distinct("_id")))
         self.assertIsNotNone(coll.aggregate([]))
         self.assertIsNotNone(coll.index_information())
@@ -231,9 +237,6 @@ class TestReadPreferences(TestReadPreferencesBase):
                           localthresholdms=-1)
 
     def test_zero_latency(self):
-        if (client_context.version >= (3, 7, 2)
-                and client_context.auth_enabled and client_context.is_rs):
-            raise SkipTest("Disabled due to SERVER-32845")
         ping_times = set()
         # Generate unique ping times.
         while len(ping_times) < len(self.client.nodes):
@@ -310,13 +313,14 @@ class TestReadPreferences(TestReadPreferencesBase):
 class ReadPrefTester(MongoClient):
     def __init__(self, *args, **kwargs):
         self.has_read_from = set()
-        client_options = client_context.ssl_client_options.copy()
+        client_options = client_context.default_client_options.copy()
         client_options.update(kwargs)
         super(ReadPrefTester, self).__init__(*args, **client_options)
 
     @contextlib.contextmanager
     def _socket_for_reads(self, read_preference):
-        context = super(ReadPrefTester, self)._socket_for_reads(read_preference)
+        context = super(ReadPrefTester, self)._socket_for_reads(
+            read_preference)
         with context as (sock_info, slave_ok):
             self.record_a_read(sock_info.address)
             yield sock_info, slave_ok
@@ -412,18 +416,13 @@ class TestCommandAndReadPreference(TestReplicaSetClientBase):
             self._test_fn(server_type, func)
 
     def test_create_collection(self):
-        # Collections should be created on primary, obviously
+        # create_collection runs listCollections on the primary to check if
+        # the collection already exists.
         self._test_primary_helper(
             lambda: self.c.pymongo_test.create_collection(
                 'some_collection%s' % random.randint(0, MAXSIZE)))
 
-    def test_drop_collection(self):
-        self._test_primary_helper(
-            lambda: self.c.pymongo_test.drop_collection('some_collection'))
-
-        self._test_primary_helper(
-            lambda: self.c.pymongo_test.some_collection.drop())
-
+    @client_context.require_version_max(4, 1, 0, -1)
     def test_group(self):
         with warnings.catch_warnings():
             warnings.simplefilter("ignore")
@@ -440,9 +439,18 @@ class TestCommandAndReadPreference(TestReplicaSetClientBase):
                                'inline_map_reduce',
                                'function() { }', 'function() { }')
 
+    @ignore_deprecations
     def test_count(self):
         self._test_coll_helper(True, self.c.pymongo_test.test, 'count')
 
+    def test_count_documents(self):
+        self._test_coll_helper(
+            True, self.c.pymongo_test.test, 'count_documents', {})
+
+    def test_estimated_document_count(self):
+        self._test_coll_helper(
+            True, self.c.pymongo_test.test, 'estimated_document_count')
+
     def test_distinct(self):
         self._test_coll_helper(True, self.c.pymongo_test.test, 'distinct', 'a')
 
