@@ -111,18 +111,13 @@ class TestReplicaSetClient(TestReplicaSetClientBase):
         c.admin.command('ping')
 
         wait_until(lambda: c.primary == self.primary, "discover primary")
+        wait_until(lambda: c.arbiters == self.arbiters, "discover arbiters")
         wait_until(lambda: c.secondaries == self.secondaries,
                    "discover secondaries")
 
-        # SERVER-32845
-        if not (client_context.version >= (3, 7, 2)
-                and client_context.auth_enabled and client_context.is_rs):
-            wait_until(lambda: c.arbiters == self.arbiters,
-                       "discover arbiters")
-            self.assertEqual(c.arbiters, self.arbiters)
-
         self.assertEqual(c.primary, self.primary)
         self.assertEqual(c.secondaries, self.secondaries)
+        self.assertEqual(c.arbiters, self.arbiters)
         self.assertEqual(c.max_pool_size, 100)
 
         # Make sure MongoClient's properties are copied to Database and
@@ -134,7 +129,7 @@ class TestReplicaSetClient(TestReplicaSetClientBase):
 
         cursor = c.pymongo_test.test.find()
         self.assertEqual(
-            ReadPreference.PRIMARY, cursor._Cursor__read_preference)
+            ReadPreference.PRIMARY, cursor._read_preference())
 
         tag_sets = [{'dc': 'la', 'rack': '2'}, {'foo': 'bar'}]
         secondary = Secondary(tag_sets=tag_sets)
@@ -155,13 +150,13 @@ class TestReplicaSetClient(TestReplicaSetClientBase):
 
         cursor = c.pymongo_test.test.find()
         self.assertEqual(
-            secondary, cursor._Cursor__read_preference)
+            secondary, cursor._read_preference())
 
         nearest = Nearest(tag_sets=[{'dc': 'ny'}, {}])
         cursor = c.pymongo_test.get_collection(
             "test", read_preference=nearest).find()
 
-        self.assertEqual(nearest, cursor._Cursor__read_preference)
+        self.assertEqual(nearest, cursor._read_preference())
         self.assertEqual(c.max_bson_size, 16777216)
         c.close()
 
@@ -229,7 +224,7 @@ class TestReplicaSetClient(TestReplicaSetClientBase):
         client.pymongo_test.test.insert_one({"dummy": u"object"})
         client.pymongo_test_bernie.test.insert_one({"dummy": u"object"})
 
-        dbs = client.database_names()
+        dbs = client.list_database_names()
         self.assertTrue("pymongo_test" in dbs)
         self.assertTrue("pymongo_test_bernie" in dbs)
         client.close()
