@@ -26,8 +26,8 @@ from bson.son import SON
 
 
 from pymongo.errors import (ConnectionFailure,
+                            OperationFailure,
                             ServerSelectionTimeoutError)
-from pymongo.monitoring import _SENSITIVE_COMMANDS
 from pymongo.mongo_client import MongoClient
 from pymongo.operations import (InsertOne,
                                 DeleteMany,
@@ -35,10 +35,13 @@ from pymongo.operations import (InsertOne,
                                 ReplaceOne,
                                 UpdateMany,
                                 UpdateOne)
+from pymongo.results import BulkWriteResult
 from pymongo.write_concern import WriteConcern
 
-from test import unittest, client_context, IntegrationTest, SkipTest
-from test.utils import rs_or_single_client, EventListener, DeprecationFilter
+from test import unittest, client_context, IntegrationTest, SkipTest, client_knobs
+from test.utils import (rs_or_single_client,
+                        DeprecationFilter,
+                        OvertCommandListener)
 from test.test_crud import check_result, run_operation
 
 # Location of JSON test specifications.
@@ -46,23 +49,6 @@ _TEST_PATH = os.path.join(
     os.path.dirname(os.path.realpath(__file__)), 'retryable_writes')
 
 
-class CommandListener(EventListener):
-    def started(self, event):
-        if event.command_name.lower() in _SENSITIVE_COMMANDS:
-            return
-        super(CommandListener, self).started(event)
-
-    def succeeded(self, event):
-        if event.command_name.lower() in _SENSITIVE_COMMANDS:
-            return
-        super(CommandListener, self).succeeded(event)
-
-    def failed(self, event):
-        if event.command_name.lower() in _SENSITIVE_COMMANDS:
-            return
-        super(CommandListener, self).failed(event)
-
-
 class TestAllScenarios(IntegrationTest):
 
     @classmethod
@@ -71,52 +57,66 @@ class TestAllScenarios(IntegrationTest):
     @client_context.require_test_commands
     def setUpClass(cls):
         super(TestAllScenarios, cls).setUpClass()
-        cls.client = rs_or_single_client(retryWrites=True)
-        cls.db = cls.client.pymongo_test
+        # Speed up the tests by decreasing the heartbeat frequency.
+        cls.knobs = client_knobs(heartbeat_frequency=0.1,
+                                 min_heartbeat_interval=0.1)
+        cls.knobs.enable()
+
+    @classmethod
+    def tearDownClass(cls):
+        cls.knobs.disable()
+        super(TestAllScenarios, cls).tearDownClass()
 
     def tearDown(self):
-        self.client.admin.command(SON([
+        client_context.client.admin.command(SON([
             ('configureFailPoint', 'onPrimaryTransactionalWrite'),
             ('mode', 'off')]))
 
     def set_fail_point(self, command_args):
         cmd = SON([('configureFailPoint', 'onPrimaryTransactionalWrite')])
         cmd.update(command_args)
-        self.client.admin.command(cmd)
+        client_context.client.admin.command(cmd)
 
 
 def create_test(scenario_def, test):
     def run_scenario(self):
         # Load data.
         assert scenario_def['data'], "tests must have non-empty data"
-        self.db.test.drop()
-        self.db.test.insert_many(scenario_def['data'])
+        client_context.client.pymongo_test.test.drop()
+        client_context.client.pymongo_test.test.insert_many(scenario_def['data'])
 
         # Set the failPoint
         self.set_fail_point(test['failPoint'])
+        self.addCleanup(self.set_fail_point, {
+            'configureFailPoint': test['failPoint']['configureFailPoint'],
+            'mode': 'off'})
 
         test_outcome = test['outcome']
         should_fail = test_outcome.get('error')
         result = None
         error = None
+
+        db = rs_or_single_client(**test['clientOptions']).pymongo_test
+        # Close the client explicitly to avoid having too many threads open.
+        self.addCleanup(db.client.close)
         try:
-            result = run_operation(self.db.test, test)
-        except ConnectionFailure as exc:
+            result = run_operation(db.test, test)
+        except (ConnectionFailure, OperationFailure) as exc:
             error = exc
 
         if should_fail:
             self.assertIsNotNone(error, 'should have raised an error')
         else:
-            self.assertIsNone(error, 'should not have raised an error')
+            self.assertIsNone(error)
 
         # Assert final state is expected.
         expected_c = test_outcome.get('collection')
         if expected_c is not None:
             expected_name = expected_c.get('name')
             if expected_name is not None:
-                db_coll = self.db[expected_name]
+                db_coll = db[expected_name]
             else:
-                db_coll = self.db.test
+                db_coll = db.test
             self.assertEqual(list(db_coll.find()), expected_c['data'])
         expected_result = test_outcome.get('result')
         # We can't test the expected result when the test should fail because
@@ -137,10 +137,26 @@ def create_tests():
                 scenario_def = json.load(scenario_stream)
 
             test_type = os.path.splitext(filename)[0]
+            min_ver, max_ver = None, None
+            if 'minServerVersion' in scenario_def:
+                min_ver = tuple(
+                    int(elt) for
+                    elt in scenario_def['minServerVersion'].split('.'))
+            if 'maxServerVersion' in scenario_def:
+                max_ver = tuple(
+                    int(elt) for
+                    elt in scenario_def['maxServerVersion'].split('.'))
 
             # Construct test from scenario.
             for test in scenario_def['tests']:
                 new_test = create_test(scenario_def, test)
+                if min_ver is not None:
+                    new_test = client_context.require_version_min(*min_ver)(
+                        new_test)
+                if max_ver is not None:
+                    new_test = client_context.require_version_max(*max_ver)(
+                        new_test)
+
                 test_name = 'test_%s_%s_%s' % (
                     dirname,
                     test_type,
@@ -240,11 +256,20 @@ class TestRetryableWrites(IgnoreDeprecationsTest):
     @classmethod
     def setUpClass(cls):
         super(TestRetryableWrites, cls).setUpClass()
-        cls.listener = CommandListener()
+        # Speed up the tests by decreasing the heartbeat frequency.
+        cls.knobs = client_knobs(heartbeat_frequency=0.1,
+                                 min_heartbeat_interval=0.1)
+        cls.knobs.enable()
+        cls.listener = OvertCommandListener()
         cls.client = rs_or_single_client(
             retryWrites=True, event_listeners=[cls.listener])
         cls.db = cls.client.pymongo_test
 
+    @classmethod
+    def tearDownClass(cls):
+        cls.knobs.disable()
+        super(TestRetryableWrites, cls).tearDownClass()
+
     def setUp(self):
         if (client_context.version.at_least(3, 5) and client_context.is_rs
                 and client_context.test_commands_enabled):
@@ -260,7 +285,7 @@ class TestRetryableWrites(IgnoreDeprecationsTest):
                 ('mode', 'off')]))
 
     def test_supported_single_statement_no_retry(self):
-        listener = CommandListener()
+        listener = OvertCommandListener()
         client = rs_or_single_client(
             retryWrites=False, event_listeners=[listener])
         self.addCleanup(client.close)
@@ -350,7 +375,7 @@ class TestRetryableWrites(IgnoreDeprecationsTest):
 
     def test_server_selection_timeout_not_retried(self):
         """A ServerSelectionTimeoutError is not retried."""
-        listener = CommandListener()
+        listener = OvertCommandListener()
         client = MongoClient(
             'somedomainthatdoesntexist.org',
             serverSelectionTimeoutMS=1,
@@ -370,7 +395,7 @@ class TestRetryableWrites(IgnoreDeprecationsTest):
         """A ServerSelectionTimeoutError on the retry attempt raises the
         original error.
         """
-        listener = CommandListener()
+        listener = OvertCommandListener()
         client = rs_or_single_client(
             retryWrites=True, event_listeners=[listener])
         self.addCleanup(client.close)
@@ -405,7 +430,7 @@ class TestRetryableWrites(IgnoreDeprecationsTest):
         coll = self.db.retryable_write_test
         coll.delete_many({})
         self.listener.results.clear()
-        coll.bulk_write([
+        bulk_result = coll.bulk_write([
             InsertOne({'_id': 1, 'l': large}),
             InsertOne({'_id': 2, 'l': large}),
             InsertOne({'_id': 3, 'l': large}),
@@ -415,8 +440,22 @@ class TestRetryableWrites(IgnoreDeprecationsTest):
             DeleteOne({'l': large}),
             DeleteOne({'l': large})])
         # Each command should fail and be retried.
-        self.assertEqual(len(self.listener.results['started']), 14)
+        # With OP_MSG 3 inserts are one batch. 2 updates another.
+        # 2 deletes a third.
+        self.assertEqual(len(self.listener.results['started']), 6)
         self.assertEqual(coll.find_one(), {'_id': 1, 'count': 1})
+        # Assert the final result
+        expected_result = {
+            "writeErrors": [],
+            "writeConcernErrors": [],
+            "nInserted": 3,
+            "nUpserted": 0,
+            "nMatched": 2,
+            "nModified": 2,
+            "nRemoved": 2,
+            "upserted": [],
+        }
+        self.assertEqual(bulk_result.bulk_api_result, expected_result)
 
     @client_context.require_version_min(3, 5)
     @client_context.require_replica_set
@@ -428,7 +467,7 @@ class TestRetryableWrites(IgnoreDeprecationsTest):
         coll.delete_many({})
         self.client.admin.command(SON([
             ('configureFailPoint', 'onPrimaryTransactionalWrite'),
-            ('mode', {'skip': 1}),
+            ('mode', {'skip': 3}),  # The number of _documents_ to skip.
             ('data', {'failBeforeCommitExceptionCode': 1})]))
         self.listener.results.clear()
         with self.client.start_session() as session:
@@ -436,7 +475,8 @@ class TestRetryableWrites(IgnoreDeprecationsTest):
             try:
                 coll.bulk_write([InsertOne({'_id': 1, 'l': large}),
                                  InsertOne({'_id': 2, 'l': large}),
-                                 InsertOne({'_id': 3, 'l': large})],
+                                 InsertOne({'_id': 3, 'l': large}),
+                                 InsertOne({'_id': 4, 'l': large})],
                                 session=session)
             except ConnectionFailure:
                 pass
