@@ -27,7 +27,6 @@ from pymongo.errors import (ConfigurationError,
                             OperationFailure)
 from pymongo.monotonic import time as _time
 from pymongo.read_concern import ReadConcern
-from pymongo.write_concern import WriteConcern
 from test import IntegrationTest, client_context, db_user, db_pwd, unittest, SkipTest
 from test.utils import ignore_deprecations, rs_or_single_client, EventListener
 
@@ -254,6 +253,7 @@ class TestSession(IntegrationTest):
             (db.command, ['ping'], {}),
             (db.create_collection, ['collection'], {}),
             (db.collection_names, [], {}),
+            (db.list_collection_names, [], {}),
             (db.validate_collection, ['collection'], {}),
             (db.drop_collection, ['collection'], {}),
             (db.current_op, [], {}),
@@ -273,24 +273,17 @@ class TestSession(IntegrationTest):
         client = self.client
         db = client.pymongo_test
 
-        extra = {'roles': ['read']}
-        if client_context.version.at_least(3, 7, 2):
-            extra['mechanisms'] = ['SCRAM-SHA-1']
-
         self._test_ops(
             client,
-            (db.add_user, ['session-test', 'pass'], extra),
+            (db.add_user, ['session-test', 'pass'], {'roles': ['read']}),
             # Do it again to test updateUser command.
-            (db.add_user, ['session-test', 'pass'], extra),
+            (db.add_user, ['session-test', 'pass'], {'roles': ['read']}),
             (db.remove_user, ['session-test'], {}))
 
-    def test_collection(self):
-        client = self.client
-        coll = client.pymongo_test.collection
-
-        # Test some collection methods - the rest are in test_cursor.
-        self._test_ops(
-            client,
+    @staticmethod
+    def collection_write_ops(coll):
+        """Generate database write ops for tests."""
+        return [
             (coll.drop, [], {}),
             (coll.bulk_write, [[InsertOne({})]], {}),
             (coll.insert_one, [{}], {}),
@@ -300,28 +293,47 @@ class TestSession(IntegrationTest):
             (coll.update_many, [{}, {'$set': {'a': 1}}], {}),
             (coll.delete_one, [{}], {}),
             (coll.delete_many, [{}], {}),
-            (coll.map_reduce, ['function() {}', 'function() {}', 'output'], {}),
-            (coll.inline_map_reduce, ['function() {}', 'function() {}'], {}),
+            (coll.map_reduce,
+             ['function() {}', 'function() {}', 'output'], {}),
             (coll.find_one_and_replace, [{}, {}], {}),
             (coll.find_one_and_update, [{}, {'$set': {'a': 1}}], {}),
             (coll.find_one_and_delete, [{}, {}], {}),
             (coll.rename, ['collection2'], {}),
             # Drop collection2 between tests of "rename", above.
-            (client.pymongo_test.drop_collection, ['collection2'], {}),
-            (coll.distinct, ['a'], {}),
-            (coll.find_one, [], {}),
-            (coll.count, [], {}),
+            (coll.database.drop_collection, ['collection2'], {}),
             (coll.create_indexes, [[IndexModel('a')]], {}),
             (coll.create_index, ['a'], {}),
             (coll.drop_index, ['a_1'], {}),
             (coll.drop_indexes, [], {}),
-            (coll.reindex, [], {}),
+            (coll.aggregate, [[{"$out": "aggout"}]], {}),
+        ]
+
+    def test_collection(self):
+        client = self.client
+        coll = client.pymongo_test.collection
+
+        # Test some collection methods - the rest are in test_cursor.
+        ops = self.collection_write_ops(coll)
+        ops.extend([
+            (coll.distinct, ['a'], {}),
+            (coll.find_one, [], {}),
+            (coll.count, [], {}),
+            (coll.count_documents, [{}], {}),
+            (coll.inline_map_reduce, ['function() {}', 'function() {}'], {}),
             (coll.list_indexes, [], {}),
             (coll.index_information, [], {}),
             (coll.options, [], {}),
-            (coll.aggregate, [[]], {}))
+            (coll.aggregate, [[]], {}),
+        ])
+
+        if client_context.supports_reindex:
+            ops.append((coll.reindex, [], {}))
+
+        self._test_ops(client, *ops)
 
     @client_context.require_no_mongos
+    @client_context.require_version_max(4, 1, 0)
+    @ignore_deprecations
     def test_parallel_collection_scan(self):
         listener = self.listener
         client = self.client
@@ -336,12 +348,9 @@ class TestSession(IntegrationTest):
                 c.batch_size(2)
                 list(c)
 
-        self._test_ops(client, (scan, [], {}))
-
-        # Implicit session with parallel_scan is uncorrelated with cursors',
-        # but each cursor's getMores all use the same lsid.
         listener.results.clear()
-        scan()
+        with client.start_session() as session:
+            scan(session)
         cursor_lsids = {}
         for event in listener.results['started']:
             self.assertIn(
@@ -683,6 +692,68 @@ class TestSession(IntegrationTest):
                                                  session=session),
             lambda cursor: list(cursor))
 
+    def _test_unacknowledged_ops(self, client, *ops):
+        listener = client.event_listeners()[0][0]
+
+        for f, args, kw in ops:
+            with client.start_session() as s:
+                listener.results.clear()
+                # In case "f" modifies its inputs.
+                args = copy.copy(args)
+                kw = copy.copy(kw)
+                kw['session'] = s
+                with self.assertRaises(
+                        ConfigurationError,
+                        msg="%s did not raise ConfigurationError" % (
+                                f.__name__,)):
+                    f(*args, **kw)
+                if f.__name__ == 'create_collection':
+                    # create_collection runs listCollections first.
+                    event = listener.results['started'].pop(0)
+                    self.assertEqual('listCollections', event.command_name)
+                    self.assertIn('lsid', event.command,
+                                  "%s sent no lsid with %s" % (
+                                     f.__name__, event.command_name))
+
+                # Should not run any command before raising an error.
+                self.assertFalse(listener.results['started'],
+                                 "%s sent command" % (f.__name__,))
+
+            self.assertTrue(s.has_ended)
+
+        # Unacknowledged write without a session does not send an lsid.
+        for f, args, kw in ops:
+            listener.results.clear()
+            f(*args, **kw)
+            self.assertGreaterEqual(len(listener.results['started']), 1)
+
+            if f.__name__ == 'create_collection':
+                # create_collection runs listCollections first.
+                event = listener.results['started'].pop(0)
+                self.assertEqual('listCollections', event.command_name)
+                self.assertIn('lsid', event.command,
+                              "%s sent no lsid with %s" % (
+                                  f.__name__, event.command_name))
+
+            for event in listener.results['started']:
+                self.assertNotIn('lsid', event.command,
+                                 "%s sent lsid with %s" % (
+                                     f.__name__, event.command_name))
+
+    def test_unacknowledged_writes(self):
+        # Ensure the collection exists.
+        self.client.pymongo_test.test_unacked_writes.insert_one({})
+        client = rs_or_single_client(w=0, event_listeners=[self.listener])
+        db = client.pymongo_test
+        coll = db.test_unacked_writes
+        ops = [
+            (client.drop_database, [db.name], {}),
+            (db.create_collection, ['collection'], {}),
+            (db.drop_collection, ['collection'], {}),
+        ]
+        ops.extend(self.collection_write_ops(coll))
+        self._test_unacknowledged_ops(client, *ops)
+
 
 class TestCausalConsistency(unittest.TestCase):
 
@@ -766,6 +837,8 @@ class TestCausalConsistency(unittest.TestCase):
             lambda coll, session: coll.find_one({}, session=session))
         self._test_reads(
             lambda coll, session: coll.count(session=session))
+        self._test_reads(
+            lambda coll, session: coll.count_documents({}, session=session))
         self._test_reads(
             lambda coll, session: coll.distinct('foo', session=session))
         self._test_reads(
@@ -774,7 +847,8 @@ class TestCausalConsistency(unittest.TestCase):
         self._test_reads(
             lambda coll, session: coll.inline_map_reduce(
                 'function() {}', 'function() {}', session=session))
-        if not client_context.is_mongos:
+        if (not client_context.is_mongos and
+                not client_context.version.at_least(4, 1, 0)):
             def scan(coll, session):
                 cursors = coll.parallel_scan(1, session=session)
                 for cur in cursors:
@@ -792,6 +866,11 @@ class TestCausalConsistency(unittest.TestCase):
             self._test_reads,
             lambda coll, session: list(
                 coll.find_raw_batches({}, session=session)))
+        self.assertRaises(
+            ConfigurationError,
+            self._test_reads,
+            lambda coll, session: coll.estimated_document_count(
+                session=session))
 
     def _test_writes(self, op):
         coll = self.client.pymongo_test.test
@@ -845,8 +924,10 @@ class TestCausalConsistency(unittest.TestCase):
             lambda coll, session: coll.drop_index("foo_1", session=session))
         self._test_writes(
             lambda coll, session: coll.drop_indexes(session=session))
-        self._test_writes(
-            lambda coll, session: coll.reindex(session=session))
+
+        if client_context.supports_reindex:
+            self._test_writes(
+                lambda coll, session: coll.reindex(session=session))
 
     def _test_no_read_concern(self, op):
         coll = self.client.pymongo_test.test
@@ -900,8 +981,6 @@ class TestCausalConsistency(unittest.TestCase):
             lambda coll, session: coll.drop_index("foo_1", session=session))
         self._test_no_read_concern(
             lambda coll, session: coll.drop_indexes(session=session))
-        self._test_no_read_concern(
-            lambda coll, session: coll.reindex(session=session))
         self._test_no_read_concern(
                 lambda coll, session: list(
                     coll.aggregate([{"$out": "aggout"}], session=session)))
@@ -916,6 +995,10 @@ class TestCausalConsistency(unittest.TestCase):
         self._test_no_read_concern(
             lambda coll, session: coll.find({}, session=session).explain())
 
+        if client_context.supports_reindex:
+            self._test_no_read_concern(
+                lambda coll, session: coll.reindex(session=session))
+
     @client_context.require_no_standalone
     def test_get_more_does_not_include_read_concern(self):
         coll = self.client.pymongo_test.test
@@ -973,13 +1056,6 @@ class TestCausalConsistency(unittest.TestCase):
             self.assertEqual(read_concern.get('level'), 'majority')
             self.assertIsNotNone(read_concern.get('afterClusterTime'))
 
-    def test_unacknowledged(self):
-        with self.client.start_session(causal_consistency=True) as s:
-            coll = self.client.pymongo_test.get_collection(
-                'test', write_concern=WriteConcern(w=0))
-            coll.insert_one({}, session=s)
-            self.assertIsNone(s.operation_time)
-
     @client_context.require_no_standalone
     def test_cluster_time_with_server_support(self):
         self.client.pymongo_test.test.insert_one({})
