@@ -1,4 +1,4 @@
-# Copyright 2014-2015 MongoDB, Inc.
+# Copyright 2014-present MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -55,6 +55,7 @@ class MockPool(object):
     def __init__(self, *args, **kwargs):
         self.pool_id = 0
         self._lock = threading.Lock()
+        self.opts = PoolOptions()
 
     def get_socket(self, all_credentials):
         return MockSocketInfo()
@@ -74,15 +75,16 @@ class MockMonitor(object):
     def __init__(self, server_description, topology, pool, topology_settings):
         self._server_description = server_description
         self._topology = topology
+        self.opened = False
 
     def open(self):
-        pass
+        self.opened = True
 
     def request_check(self):
         pass
 
     def close(self):
-        pass
+        self.opened = False
 
 
 class SetNameDiscoverySettings(TopologySettings):
@@ -121,9 +123,16 @@ def disconnected(topology, server_address):
     topology.on_change(ServerDescription(server_address))
 
 
+def get_server(topology, hostname):
+    return topology.get_server_by_address((hostname, 27017))
+
+
 def get_type(topology, hostname):
-    description = topology.get_server_by_address((hostname, 27017)).description
-    return description.server_type
+    return get_server(topology, hostname).description.server_type
+
+
+def get_monitor(topology, hostname):
+    return get_server(topology, hostname)._monitor
 
 
 class TopologyTest(unittest.TestCase):
@@ -173,35 +182,41 @@ class TestSingleServerTopology(TopologyTest):
                 'ok': 1,
                 'ismaster': True,
                 'hosts': ['a'],
-                'setName': 'rs'}),
+                'setName': 'rs',
+                'maxWireVersion': 6}),
 
             (SERVER_TYPE.RSSecondary, {
                 'ok': 1,
                 'ismaster': False,
                 'secondary': True,
                 'hosts': ['a'],
-                'setName': 'rs'}),
+                'setName': 'rs',
+                'maxWireVersion': 6}),
 
             (SERVER_TYPE.Mongos, {
                 'ok': 1,
                 'ismaster': True,
-                'msg': 'isdbgrid'}),
+                'msg': 'isdbgrid',
+                'maxWireVersion': 6}),
 
             (SERVER_TYPE.RSArbiter, {
                 'ok': 1,
                 'ismaster': False,
                 'arbiterOnly': True,
                 'hosts': ['a'],
-                'setName': 'rs'}),
+                'setName': 'rs',
+                'maxWireVersion': 6}),
 
             (SERVER_TYPE.Standalone, {
                 'ok': 1,
-                'ismaster': True}),
+                'ismaster': True,
+                'maxWireVersion': 6}),
 
             # Slave.
             (SERVER_TYPE.Standalone, {
                 'ok': 1,
-                'ismaster': False}),
+                'ismaster': False,
+                'maxWireVersion': 6}),
         ]:
             t = create_mock_topology()
 
@@ -221,6 +236,15 @@ class TestSingleServerTopology(TopologyTest):
             s = t.select_server(writable_server_selector)
             self.assertEqual(server_type, s.description.server_type)
 
+            # Topology type single is always readable and writable regardless
+            # of server type or state.
+            self.assertEqual(t.description.topology_type_name, 'Single')
+            self.assertTrue(t.description.has_writable_server())
+            self.assertTrue(t.description.has_readable_server())
+            self.assertTrue(t.description.has_readable_server(Secondary()))
+            self.assertTrue(t.description.has_readable_server(
+                Secondary(tag_sets=[{'tag': 'does-not-exist'}])))
+
     def test_reopen(self):
         t = create_mock_topology()
 
@@ -238,9 +262,10 @@ class TestSingleServerTopology(TopologyTest):
         available = True
 
         class TestMonitor(Monitor):
-            def _check_with_socket(self, sock_info):
+            def _check_with_socket(self, *args, **kwargs):
                 if available:
-                    return IsMaster({'ok': 1}), round_trip_time
+                    return (IsMaster({'ok': 1, 'maxWireVersion': 6}),
+                            round_trip_time)
                 else:
                     raise AutoReconnect('mock monitor error')
 
@@ -289,6 +314,81 @@ class TestSingleServerTopology(TopologyTest):
 
 
 class TestMultiServerTopology(TopologyTest):
+    def test_readable_writable(self):
+        t = create_mock_topology(replica_set_name='rs')
+        got_ismaster(t, ('a', 27017), {
+            'ok': 1,
+            'ismaster': True,
+            'setName': 'rs',
+            'hosts': ['a', 'b']})
+
+        got_ismaster(t, ('b', 27017), {
+            'ok': 1,
+            'ismaster': False,
+            'secondary': True,
+            'setName': 'rs',
+            'hosts': ['a', 'b']})
+
+        self.assertTrue(
+            t.description.topology_type_name, 'ReplicaSetWithPrimary')
+        self.assertTrue(t.description.has_writable_server())
+        self.assertTrue(t.description.has_readable_server())
+        self.assertTrue(
+            t.description.has_readable_server(Secondary()))
+        self.assertFalse(
+            t.description.has_readable_server(
+                Secondary(tag_sets=[{'tag': 'exists'}])))
+
+        t = create_mock_topology(replica_set_name='rs')
+        got_ismaster(t, ('a', 27017), {
+            'ok': 1,
+            'ismaster': False,
+            'secondary': False,
+            'setName': 'rs',
+            'hosts': ['a', 'b']})
+
+        got_ismaster(t, ('b', 27017), {
+            'ok': 1,
+            'ismaster': False,
+            'secondary': True,
+            'setName': 'rs',
+            'hosts': ['a', 'b']})
+
+        self.assertTrue(
+            t.description.topology_type_name, 'ReplicaSetNoPrimary')
+        self.assertFalse(t.description.has_writable_server())
+        self.assertFalse(t.description.has_readable_server())
+        self.assertTrue(
+            t.description.has_readable_server(Secondary()))
+        self.assertFalse(
+            t.description.has_readable_server(
+                Secondary(tag_sets=[{'tag': 'exists'}])))
+
+        t = create_mock_topology(replica_set_name='rs')
+        got_ismaster(t, ('a', 27017), {
+            'ok': 1,
+            'ismaster': True,
+            'setName': 'rs',
+            'hosts': ['a', 'b']})
+
+        got_ismaster(t, ('b', 27017), {
+            'ok': 1,
+            'ismaster': False,
+            'secondary': True,
+            'setName': 'rs',
+            'hosts': ['a', 'b'],
+            'tags': {'tag': 'exists'}})
+
+        self.assertTrue(
+            t.description.topology_type_name, 'ReplicaSetWithPrimary')
+        self.assertTrue(t.description.has_writable_server())
+        self.assertTrue(t.description.has_readable_server())
+        self.assertTrue(
+            t.description.has_readable_server(Secondary()))
+        self.assertTrue(
+            t.description.has_readable_server(
+                Secondary(tag_sets=[{'tag': 'exists'}])))
+
     def test_close(self):
         t = create_mock_topology(replica_set_name='rs')
         got_ismaster(t, ('a', 27017), {
@@ -306,6 +406,8 @@ class TestMultiServerTopology(TopologyTest):
 
         self.assertEqual(SERVER_TYPE.RSPrimary, get_type(t, 'a'))
         self.assertEqual(SERVER_TYPE.RSSecondary, get_type(t, 'b'))
+        self.assertTrue(get_monitor(t, 'a').opened)
+        self.assertTrue(get_monitor(t, 'b').opened)
         self.assertEqual(TOPOLOGY_TYPE.ReplicaSetWithPrimary,
                          t.description.topology_type)
 
@@ -313,19 +415,28 @@ class TestMultiServerTopology(TopologyTest):
         self.assertEqual(2, len(t.description.server_descriptions()))
         self.assertEqual(SERVER_TYPE.Unknown, get_type(t, 'a'))
         self.assertEqual(SERVER_TYPE.Unknown, get_type(t, 'b'))
+        self.assertFalse(get_monitor(t, 'a').opened)
+        self.assertFalse(get_monitor(t, 'b').opened)
         self.assertEqual('rs', t.description.replica_set_name)
         self.assertEqual(TOPOLOGY_TYPE.ReplicaSetNoPrimary,
                          t.description.topology_type)
 
+        # A closed topology should not be updated when receiving an isMaster.
         got_ismaster(t, ('a', 27017), {
             'ok': 1,
             'ismaster': True,
             'setName': 'rs',
-            'hosts': ['a', 'b']})
+            'hosts': ['a', 'b', 'c']})
 
-        self.assertEqual(SERVER_TYPE.RSPrimary, get_type(t, 'a'))
+        self.assertEqual(2, len(t.description.server_descriptions()))
+        self.assertEqual(SERVER_TYPE.Unknown, get_type(t, 'a'))
         self.assertEqual(SERVER_TYPE.Unknown, get_type(t, 'b'))
-        self.assertEqual(TOPOLOGY_TYPE.ReplicaSetWithPrimary,
+        self.assertFalse(get_monitor(t, 'a').opened)
+        self.assertFalse(get_monitor(t, 'b').opened)
+        # Server c should not have been added.
+        self.assertEqual(None, get_server(t, 'c'))
+        self.assertEqual('rs', t.description.replica_set_name)
+        self.assertEqual(TOPOLOGY_TYPE.ReplicaSetNoPrimary,
                          t.description.topology_type)
 
     def test_reset_server(self):
@@ -388,7 +499,7 @@ class TestMultiServerTopology(TopologyTest):
         self.assertEqual(t.description.replica_set_name, None)
         self.assertEqual(t.description.topology_type,
                          TOPOLOGY_TYPE.ReplicaSetNoPrimary)
-
+        t.open()
         got_ismaster(t, address, {
             'ok': 1,
             'ismaster': True,
@@ -424,7 +535,7 @@ class TestMultiServerTopology(TopologyTest):
         self.assertEqual(t.description.replica_set_name, None)
         self.assertEqual(t.description.topology_type,
                          TOPOLOGY_TYPE.ReplicaSetNoPrimary)
-
+        t.open()
         got_ismaster(t, address, {
             'ok': 1,
             'ismaster': False,
@@ -475,8 +586,33 @@ class TestMultiServerTopology(TopologyTest):
             t.select_servers(any_server_selector)
         except ConfigurationError as e:
             # Error message should say which server failed and why.
-            self.assertTrue('a:27017' in str(e))
-            self.assertTrue('wire protocol versions 11 through 12' in str(e))
+            self.assertEqual(
+                str(e),
+                "Server at a:27017 requires wire version 11, but this version "
+                "of PyMongo only supports up to %d."
+                % (common.MAX_SUPPORTED_WIRE_VERSION,))
+        else:
+            self.fail('No error with incompatible wire version')
+
+        # Incompatible.
+        got_ismaster(t, address, {
+            'ok': 1,
+            'ismaster': True,
+            'setName': 'rs',
+            'hosts': ['a'],
+            'minWireVersion': 0,
+            'maxWireVersion': 0})
+
+        try:
+            t.select_servers(any_server_selector)
+        except ConfigurationError as e:
+            # Error message should say which server failed and why.
+            self.assertEqual(
+                str(e),
+                "Server at a:27017 reports wire version 0, but this version "
+                "of PyMongo requires at least %d (MongoDB %s)."
+                % (common.MIN_SUPPORTED_WIRE_VERSION,
+                   common.MIN_SUPPORTED_SERVER_VERSION))
         else:
             self.fail('No error with incompatible wire version')
 
@@ -492,6 +628,7 @@ class TestMultiServerTopology(TopologyTest):
             'ismaster': True,
             'setName': 'rs',
             'hosts': ['a', 'b'],
+            'maxWireVersion': 6,
             'maxWriteBatchSize': 1})
 
         got_ismaster(t, ('b', 27017), {
@@ -500,6 +637,7 @@ class TestMultiServerTopology(TopologyTest):
             'secondary': True,
             'setName': 'rs',
             'hosts': ['a', 'b'],
+            'maxWireVersion': 6,
             'maxWriteBatchSize': 2})
 
         # Uses primary's max batch size.
@@ -511,6 +649,7 @@ class TestMultiServerTopology(TopologyTest):
             'ismaster': True,
             'setName': 'rs',
             'hosts': ['a', 'b'],
+            'maxWireVersion': 6,
             'maxWriteBatchSize': 2})
 
         self.assertEqual(2, write_batch_size())
@@ -524,6 +663,7 @@ def wait_for_master(topology):
     thread. In applications this is harmless but it would break some tests,
     so we pass server_selection_timeout=0 and poll instead.
     """
+
     def get_master():
         try:
             return topology.select_server(writable_server_selector, 0)
@@ -541,10 +681,10 @@ class TestTopologyErrors(TopologyTest):
         ismaster_count = [0]
 
         class TestMonitor(Monitor):
-            def _check_with_socket(self, sock_info):
+            def _check_with_socket(self, *args, **kwargs):
                 ismaster_count[0] += 1
                 if ismaster_count[0] == 1:
-                    return IsMaster({'ok': 1}), 0
+                    return IsMaster({'ok': 1, 'maxWireVersion': 6}), 0
                 else:
                     raise AutoReconnect('mock monitor error')
 
@@ -562,10 +702,10 @@ class TestTopologyErrors(TopologyTest):
         ismaster_count = [0]
 
         class TestMonitor(Monitor):
-            def _check_with_socket(self, sock_info):
+            def _check_with_socket(self, *args, **kwargs):
                 ismaster_count[0] += 1
                 if ismaster_count[0] in (1, 3):
-                    return IsMaster({'ok': 1}), 0
+                    return IsMaster({'ok': 1, 'maxWireVersion': 6}), 0
                 else:
                     raise AutoReconnect('mock monitor error')
 
@@ -584,7 +724,7 @@ class TestTopologyErrors(TopologyTest):
         exception = AssertionError('internal error')
 
         class TestMonitor(Monitor):
-            def _check_with_socket(self, sock_info):
+            def _check_with_socket(self, *args, **kwargs):
                 raise exception
 
         t = create_mock_topology(monitor_class=TestMonitor)
@@ -625,12 +765,12 @@ class TestServerSelectionErrors(TopologyTest):
 
         self.assertMessage(
             'No replica set members match selector'
-            ' "Secondary(tag_sets=None)"',
+            ' "Secondary(tag_sets=None, max_staleness=-1)"',
             t, ReadPreference.SECONDARY)
 
         self.assertMessage(
             "No replica set members match selector"
-            " \"Secondary(tag_sets=[{'dc': 'ny'}])\"",
+            " \"Secondary(tag_sets=[{'dc': 'ny'}], max_staleness=-1)\"",
             t, Secondary(tag_sets=[{'dc': 'ny'}]))
 
     def test_bad_replica_set_name(self):
