@@ -1,4 +1,4 @@
-# Copyright 2011-2015 MongoDB, Inc.
+# Copyright 2011-present MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -332,7 +332,15 @@ class TestURI(unittest.TestCase):
                                    "test.yield_historical.in"))
 
         res = copy.deepcopy(orig)
-        res['options'] = {'readpreference': ReadPreference.SECONDARY.mode}
+        res['database'] = 'test'
+        res['collection'] = 'name/with "delimiters'
+        self.assertEqual(
+            res, parse_uri("mongodb://localhost/test.name/with \"delimiters"))
+
+        res = copy.deepcopy(orig)
+        res['options'] = {
+            'readpreference': ReadPreference.SECONDARY.mongos_mode
+        }
         self.assertEqual(res, parse_uri(
             "mongodb://localhost/?readPreference=secondary"))
 
@@ -389,10 +397,13 @@ class TestURI(unittest.TestCase):
                                    "@localhost/foo?authMechanism=GSSAPI"))
 
         res = copy.deepcopy(orig)
-        res['options'] = {'readpreference': ReadPreference.SECONDARY.mode,
-                          'readpreferencetags': [
-                              {'dc': 'west', 'use': 'website'},
-                              {'dc': 'east', 'use': 'website'}]}
+        res['options'] = {
+            'readpreference': ReadPreference.SECONDARY.mongos_mode,
+            'readpreferencetags': [
+                {'dc': 'west', 'use': 'website'},
+                {'dc': 'east', 'use': 'website'}
+            ]
+        }
         res['username'] = 'user@domain.com'
         res['password'] = 'password'
         res['database'] = 'foo'
@@ -403,11 +414,14 @@ class TestURI(unittest.TestCase):
                                    "readpreferencetags=dc:east,use:website"))
 
         res = copy.deepcopy(orig)
-        res['options'] = {'readpreference': ReadPreference.SECONDARY.mode,
-                          'readpreferencetags': [
-                              {'dc': 'west', 'use': 'website'},
-                              {'dc': 'east', 'use': 'website'},
-                              {}]}
+        res['options'] = {
+            'readpreference': ReadPreference.SECONDARY.mongos_mode,
+            'readpreferencetags': [
+                {'dc': 'west', 'use': 'website'},
+                {'dc': 'east', 'use': 'website'},
+                {}
+            ]
+        }
         res['username'] = 'user@domain.com'
         res['password'] = 'password'
         res['database'] = 'foo'
@@ -446,6 +460,30 @@ class TestURI(unittest.TestCase):
         for key in res['options']:
             self.assertTrue(isinstance(key, str))
 
+    def test_parse_ssl_paths(self):
+        # Turn off "validate" since these paths don't exist on filesystem.
+        self.assertEqual(
+            {'collection': None,
+             'database': None,
+             'nodelist': [('/MongoDB.sock', None)],
+             'options': {'ssl_certfile': '/a/b'},
+             'password': 'foo/bar',
+             'username': 'jesse'},
+            parse_uri(
+                'mongodb://jesse:foo%2Fbar@%2FMongoDB.sock/?ssl_certfile=/a/b',
+                validate=False))
+
+        self.assertEqual(
+            {'collection': None,
+             'database': None,
+             'nodelist': [('/MongoDB.sock', None)],
+             'options': {'ssl_certfile': 'a/b'},
+             'password': 'foo/bar',
+             'username': 'jesse'},
+            parse_uri(
+                'mongodb://jesse:foo%2Fbar@%2FMongoDB.sock/?ssl_certfile=a/b',
+                validate=False))
+
 
 if __name__ == "__main__":
     unittest.main()
