@@ -29,12 +29,11 @@ from functools import partial
 
 from pymongo import MongoClient, monitoring
 from pymongo.errors import AutoReconnect, OperationFailure
+from pymongo.monitoring import _SENSITIVE_COMMANDS
 from pymongo.server_selectors import (any_server_selector,
                                       writable_server_selector)
 from pymongo.write_concern import WriteConcern
-from test import (client_context,
-                  db_user,
-                  db_pwd)
+from test import (client_context, db_user, db_pwd)
 
 
 IMPOSSIBLE_WRITE_CONCERN = WriteConcern(w=1000)
@@ -74,6 +73,21 @@ class EventListener(monitoring.CommandListener):
         self.results['failed'].append(event)
 
 
+class OvertCommandListener(EventListener):
+    """A CommandListener that ignores sensitive commands."""
+    def started(self, event):
+        if event.command_name.lower() not in _SENSITIVE_COMMANDS:
+            super(OvertCommandListener, self).started(event)
+
+    def succeeded(self, event):
+        if event.command_name.lower() not in _SENSITIVE_COMMANDS:
+            super(OvertCommandListener, self).succeeded(event)
+
+    def failed(self, event):
+        if event.command_name.lower() not in _SENSITIVE_COMMANDS:
+            super(OvertCommandListener, self).failed(event)
+
+
 class ServerAndTopologyEventListener(monitoring.ServerListener,
                                      monitoring.TopologyListener):
     """Listens to all events."""
@@ -111,15 +125,16 @@ def _connection_string(h, p, authenticate):
     if h.startswith("mongodb://"):
         return h
     elif client_context.auth_enabled and authenticate:
-        return "mongodb://%s:%s@%s:%d" % (db_user, db_pwd, h, p)
+        return "mongodb://%s:%s@%s:%d" % (db_user, db_pwd, str(h), p)
     else:
-        return "mongodb://%s:%d" % (h, p)
+        return "mongodb://%s:%d" % (str(h), p)
 
 
 def _mongo_client(host, port, authenticate=True, direct=False, **kwargs):
     """Create a new client over SSL/TLS if necessary."""
-    client_options = client_context.ssl_client_options.copy()
-
+    host = host or client_context.host
+    port = port or client_context.port
+    client_options = client_context.default_client_options.copy()
     if client_context.replica_set_name and not direct:
         client_options['replicaSet'] = client_context.replica_set_name
     client_options.update(kwargs)
@@ -130,32 +145,27 @@ def _mongo_client(host, port, authenticate=True, direct=False, **kwargs):
     return client
 
 
-def single_client_noauth(
-        h=client_context.host, p=client_context.port, **kwargs):
+def single_client_noauth(h=None, p=None, **kwargs):
     """Make a direct connection. Don't authenticate."""
     return _mongo_client(h, p, authenticate=False, direct=True, **kwargs)
 
 
-def single_client(
-        h=client_context.host, p=client_context.port, **kwargs):
+def single_client(h=None, p=None, **kwargs):
     """Make a direct connection, and authenticate if necessary."""
     return _mongo_client(h, p, direct=True, **kwargs)
 
 
-def rs_client_noauth(
-        h=client_context.host, p=client_context.port, **kwargs):
+def rs_client_noauth(h=None, p=None, **kwargs):
     """Connect to the replica set. Don't authenticate."""
     return _mongo_client(h, p, authenticate=False, **kwargs)
 
 
-def rs_client(
-        h=client_context.host, p=client_context.port, **kwargs):
+def rs_client(h=None, p=None, **kwargs):
     """Connect to the replica set and authenticate if necessary."""
     return _mongo_client(h, p, **kwargs)
 
 
-def rs_or_single_client_noauth(
-        h=client_context.host, p=client_context.port, **kwargs):
+def rs_or_single_client_noauth(h=None, p=None, **kwargs):
     """Connect to the replica set if there is one, otherwise the standalone.
 
     Like rs_or_single_client, but does not authenticate.
@@ -163,8 +173,7 @@ def rs_or_single_client_noauth(
     return _mongo_client(h, p, authenticate=False, **kwargs)
 
 
-def rs_or_single_client(
-        h=client_context.host, p=client_context.port, **kwargs):
+def rs_or_single_client(h=None, p=None, **kwargs):
     """Connect to the replica set if there is one, otherwise the standalone.
 
     Authenticates if necessary.
@@ -258,7 +267,7 @@ def server_is_master_with_slave(client):
 
 
 def drop_collections(db):
-    for coll in db.collection_names():
+    for coll in db.list_collection_names():
         if not coll.startswith('system'):
             db.drop_collection(coll)
 
@@ -471,7 +480,7 @@ def run_threads(collection, target):
         t.start()
 
     for t in threads:
-        t.join(30)
+        t.join(60)
         assert not t.isAlive()
 
 
@@ -545,3 +554,19 @@ def eventlet_monkey_patched():
 
 def is_greenthread_patched():
     return gevent_monkey_patched() or eventlet_monkey_patched()
+
+
+def disable_replication(client):
+    """Disable replication on all secondaries, requires MongoDB 3.2."""
+    for host, port in client.secondaries:
+        secondary = single_client(host, port)
+        secondary.admin.command('configureFailPoint', 'stopReplProducer',
+                                mode='alwaysOn')
+
+
+def enable_replication(client):
+    """Enable replication on all secondaries, requires MongoDB 3.2."""
+    for host, port in client.secondaries:
+        secondary = single_client(host, port)
+        secondary.admin.command('configureFailPoint', 'stopReplProducer',
+                                mode='off')
